#include "animation.h"
#include <fstream>
#include <string>

namespace AntiKill
{

Animation::Animation()
{
	actions.clear();
	action = frame = 0;
	current = 0;
	direction = DIR_RIGHT;
	loop = true;
}

Animation::~Animation()
{
	for(vector<AniAction*>::iterator it=actions.begin(); it!=actions.end(); it++){
		delete *it;
	}
	actions.clear();
}

}

void AntiKill::Animation::draw(BITMAP* dest, int x, int y)
{
	if(!current)
		return;
	
	if(direction==DIR_LEFT)
		draw_sprite_h_flip(dest, current->sub[frame], x, y);
	else
		draw_sprite(dest, current->sub[frame], x, y);
}

int AntiKill::Animation::getAction()
{
	return action;
}

int AntiKill::Animation::getFrame()
{
	return frame;
}

int AntiKill::Animation::getFrameHeight() 
{
	return current->sub[frame]->h;
}

int AntiKill::Animation::getFrameWidth() 
{
	return current->sub[frame]->w;
}

void AntiKill::Animation::load(const char* file)
{
	ifstream ifs(file);
	string picfile;
	if(ifs.bad())
		return;
	ifs >> actionCount;
	for(int i=0; i<actionCount; i++){
		int index, rate, lp;
		ifs >> index >> rate >> lp;
		ifs >> picfile;
		BITMAP* bmp = load_bitmap(picfile.c_str(), 0);
		if(!bmp){
			allegro_message("failed to load bitmap: %s", picfile.c_str());
			return;
		}
		AniAction* a = new AniAction;
		a->index = index;
		a->frameRate = rate;
		a->loop = (bool)lp;
		a->pic = bmp;
		ifs >> a->frameCount;
		for(int j=0; j<a->frameCount; j++){
			int x1, y1, x2, y2;
			ifs >> x1 >> y1 >> x2 >> y2;
			a->sub[j] = create_sub_bitmap(a->pic, x1, y1, x2-x1, y2-y1);
			if(!a->sub[j]){
				allegro_message("failed to create sub bitmap for %s", picfile.c_str());
			}
		}
		actions.push_back(a);
	}
	ifs.close();
}

void AntiKill::Animation::setAction(int i)
{
	for(vector<AniAction*>::iterator it=actions.begin(); it!=actions.end(); it++){
		AniAction* a = *it;
		if(a->index==i){
			current = a;
			action = i;
			frame = 0;
			frameRate = a->frameRate;
			loop = a->loop;
			timer = 0;
			return;
		}
	}
}

void AntiKill::Animation::setFrame(int i)
{
	frame = i;
	if( frame >= current->frameCount || frame <0)
		frame = 0;
}

void AntiKill::Animation::nextFrame()
{
	if(!current || current->frameCount==0)
		return;
	if(timer == 0){
		if( frame < current->frameCount - 1)
			frame ++;
		else if(loop)
			frame = 0;
		timer = frameRate;
	}else{
		timer --;
	}
}

void AntiKill::Animation::setDirection(int d)
{
	direction = d;
}

int AntiKill::Animation::getDirection()
{
	return direction;
}

void AntiKill::Animation::setAction(int i, bool l)
{
	setAction(i);
	loop = l;
}

