#include "blocksystem.h"
#include "game.h"
#include <iostream>
using namespace std;

namespace AntiKill
{

BlockSystem::BlockSystem()
{
	blockWidth = 200;
	blockHeight = 200;
	rows = cols = 0;
}

BlockSystem::~BlockSystem()
{
}

}

void AntiKill::BlockSystem::addObject(Object* o)
{
	const Rect& r = o->getRectangle();
	this->addObject(o, r.x1, r.y1, r.x2, r.y2);
}

bool AntiKill::BlockSystem::checkCollision(Object* o)
{
	const Rect& r = o->getRectangle();
	this->checkCollision(o, r.x1, r.y1, r.x2, r.y2);
}

void AntiKill::BlockSystem::cleanup()
{
}

void AntiKill::BlockSystem::initialize(int mapw, int maph)
{
	if( rows ){
		for( int j=0; j<rows; j++ )
			delete[] blocks[j];
		delete[] blocks;
		rows = cols = 0;
		destroy_bitmap(blockMap);
	}
	cols = mapw / blockWidth + 2;
	rows = maph / blockHeight + 2;
	blocks = new vector<Object*>*[rows];
	for(int i=0; i<rows; i++)
		blocks[i] = new vector<Object*>[cols];
	blockMap = create_bitmap(mapw, maph);
	clear_bitmap(blockMap);
	blockColor = makecol(255,255,255);
	iterating = false;
}

void AntiKill::BlockSystem::removeObject(Object* o)
{
	const Rect& r = o->getRectangle();
	this->removeObject(o, r.x1, r.y1, r.x2, r.y2);
}

void AntiKill::BlockSystem::addObject(Object* o, int x1, int y1, int x2, int y2)
{
	if(!o->isCollidable())
		return ;
	if(iterating){
		allegro_message("You should not addObject while iterating.");
		return;
	}
	for(unsigned int i=y1/blockHeight; i<=y2/blockHeight && i<rows; i++)
		for(unsigned int j=x1/blockWidth; j<=x2/blockWidth && j<cols; j++){
//			cout << "add to block "<<i<<","<<j<<endl;
			vector<Object*>& v = blocks[i][j];
			v.push_back(o);
		}
	Player* p = dynamic_cast<Player*>(o);
	if(p)
		rect(blockMap, x1, y1, x2-1, y2-1, p->getColor());
	else
		rect(blockMap, x1, y1, x2-1, y2-1, blockColor);
}

void AntiKill::BlockSystem::removeObject(Object* o, int x1, int y1, int x2, int y2)
{
	if(!o->isCollidable())
		return ;
	if(iterating){
		allegro_message("You should not removeObject while iterating.");
		return;
	}
	for(unsigned int i=y1/blockHeight; i<=y2/blockHeight && i<rows; i++)
		for(unsigned int j=x1/blockWidth; j<=x2/blockWidth && j<cols; j++){
			vector<Object*>& v = blocks[i][j];
			for(vector<Object*>::iterator it=v.begin(); it!=v.end(); it++ )
				if(*it==o){
					v.erase(it);
//					cout << "remove from block " << i << ","<<j<<endl;
					break;
				}
		}
	rect(blockMap, x1, y1, x2-1, y2-1, makecol(0,0,0));
}

bool AntiKill::BlockSystem::checkCollision(Object* o, int x1, int y1, int x2, int y2)
{
	bool collision = false;
	iterating = true;
	for(unsigned int i=y1/blockHeight; i<=y2/blockHeight && i<rows; i++)
		for(unsigned int j=x1/blockWidth; j<=x2/blockWidth && j<cols; j++){
			vector<Object*>& v = blocks[i][j];
			for(vector<Object*>::iterator it=v.begin(); it!=v.end(); it++ )
				if( o!=*it && o->collide(*it) )
					collision = true;
		}
	iterating = false;
	return collision;
}

BITMAP* AntiKill::BlockSystem::getBlockMap()
{
	return blockMap;
}

struct CollisionPoint{
	bool collision;
	int x;
	int y;
	int selfColor;
};
static void checkPoint(BITMAP *bmp, int x, int y, int d)
{
	CollisionPoint *p = (CollisionPoint*)d;
	if( !p->collision ){
		int col = getpixel(bmp, x, y);
		if( col && col !=p->selfColor ){
			p->collision = true;
			p->x = x;
			p->y = y;
		}
	}
}

bool AntiKill::BlockSystem::checkLineCollision(int col, int x1, int y1, int x2, int y2, int* cx, int* cy)
{
	CollisionPoint cp;
	cp.collision = false;
	cp.selfColor = col;
	do_line(blockMap, x1, y1, x2, y2, (int)&cp, &checkPoint );
	if(cp.collision){
		*cx = cp.x;
		*cy = cp.y;
		return true;
	}
	return false;
}

void AntiKill::BlockSystem::draw(BITMAP* dest, int x, int y)
{
	blit(blockMap, dest, x, y, 0, 0, dest->w, dest->h );
}

AntiKill::Object* AntiKill::BlockSystem::getObjectByPosition(int x, int y)
{
	Object* found = 0;
	iterating = true;
	unsigned i = y/blockHeight, j= x/blockWidth;
	if(i<rows && j<cols){
		vector<Object*>& v = blocks[i][j];
		for(vector<Object*>::iterator it=v.begin(); it!=v.end(); it++ ){
			Object* o = *it;
			if(o->getX()<=x && o->getX()+o->getWidth()>=x && 
				o->getY()<=y && o->getY()+o->getHeight()>=y ){
					found = o;
					break;
				}
		}
	}
	iterating = false;
	return found;
}

