#include <iostream>
#include "dialog.h"
#include "game.h"


namespace AntiKill
{

Dialog::Dialog()
{
	this->bitmap = 0;
	this->visible = false;
	this->delay = 300;
	this->cellSpace = 8;
	this->margin = 10;
}

Dialog::~Dialog()
{
	if(this->bitmap){
		destroy_bitmap(this->bitmap);
		this->bitmap = 0;
	}
}

}

void AntiKill::Dialog::draw(BITMAP* dest, int cx, int cy)
{
	if(this->visible&&this->bitmap){
		int wx = this->x - cx;
		int wy = this->y - cy;
		set_trans_blender(0, 0, 0, 200);
		draw_trans_sprite(dest, bitmap, wx, wy);
	}
}

const string& AntiKill::Dialog::getText() const
{
	return this->message; 
}

AntiKill::Dialog& AntiKill::Dialog::hide()
{
	if(this->delay-this->counter<30)
		return *this;
	this->visible = false;
	return *this;
}

void AntiKill::Dialog::move()
{
	if(this->visible){
		this->counter --;
		if(this->counter < 0)
			this->visible = false;
	}
}

AntiKill::Dialog& AntiKill::Dialog::setDelay(int t)
{
	this->delay = t;
	return *this;
}

AntiKill::Dialog& AntiKill::Dialog::setPosition(int x, int y)
{
	this->x = x;
	this->y = y;
	return *this;
}

AntiKill::Dialog& AntiKill::Dialog::setText(string text)
{
	int maxlen = 0;
	this->message = text;
	lines.clear();
	for(int i=0; i<=text.length(); i++ ){
		if(text[i]=='\n' || text[i]=='\0'){
			text[i] = '\0';
			int len = text_length(font, text.c_str());
			if(len > maxlen)
				maxlen = len;
			lines.push_back(text.c_str());
			text.erase(0, i+1);
			i = 0;
		}
	}
	w = maxlen + cellSpace*2 + margin*2;
	h = lines.size() * (text_height(font)+cellSpace) + cellSpace + margin*2;
	this->bgColor = makecol(0,0,0);
	this->textColor = makecol( 255, 255, 255 );
	build();
	return *this;
}

AntiKill::Dialog& AntiKill::Dialog::show()
{
	this->visible = true;
	this->counter = this->delay;
	return *this;
}

void AntiKill::Dialog::build()
{
	if(bitmap)
		destroy_bitmap(bitmap);
	bitmap = create_bitmap(w, h);
	clear_to_color(bitmap, makecol(255,0,255));
	if(!bitmap)
		return;
	int wx=margin;
	int wy=margin;
	rectfill( bitmap, wx, wy, w-1-margin, h-1-margin, this->bgColor );
	wx += cellSpace;
	wy += cellSpace;
	int fh = text_height(font);
	for(vector<string>::iterator it=lines.begin(); it!=lines.end(); it++ ){
		string& s = *it;
		textout_ex(bitmap, font, s.c_str(), wx, wy, this->textColor, -1 );
		wy += fh + cellSpace;
	}
	Game& g = Game::getInstance();
	y -= h;
	if( x + w - g.getMapSystem().getClientX() > g.getMapSystem().getClientWidth() ){
		x -= w;
		triangle(bitmap, w-1, h-1, w-margin, h-margin-cellSpace, w-margin-cellSpace, h-margin, this->bgColor );
	}else{
		triangle(bitmap, 0, h-1, margin, h-margin-cellSpace, margin+cellSpace, h-margin, this->bgColor );
	}
}

AntiKill::Dialog& AntiKill::Dialog::setColor(int c)
{
	return *this;
}

bool AntiKill::Dialog::isVisible() const
{
	return this->visible;
}

