#include <allegro.h>
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <ctime>
#include "aiplayer.h"
#include "game.h"
#include "lock.h"

namespace AntiKill
{
static Game*	instance;

Game::Game()
{
	allegro_init();
	install_timer();
	inputSystem.initialize();
	soundSystem.initialize();
	instance = this;
	srand( time(0)) ;
	initializeLock();

	this->main = 0;
	end = false;
}

Game::~Game()
{
	allegro_exit();
	destroyLock();
}

}

bool AntiKill::Game::start()
{
	if (!screen.setVideoMode(800,600,32))
		if (!screen.setVideoMode(640,480,32))
			return false;
	mapSystem.setClientWidth(screen.getWidth()).setClientHeight(screen.getHeight());
	infoBar.initialize();
	infoBar.show();
	robot = new Robot();
	robot->setX(10);
	robot->setY(150);
	scriptSystem.start();
	gameLoop();
	scriptSystem.stop();
	delete robot;
	return true;
}

void AntiKill::Game::gameLoop()
{
	BITMAP* buffer;
	time_t s = time(NULL);
	int f=0,fps=0, fpsControl=60, restMs=10;
	while (!end) {
		if (key[KEY_ESC]) {
			this->end = true;
		}
		if (time(NULL)-s>=1) {
			fps=f;
			f=0;
			s=time(NULL);
			if(fps<fpsControl)
				restMs -= 1;
			else
				restMs += 1;
		}
		f++;
		buffer = screen.getBufferPage();
		//do input
		inputSystem.doInput();

		lockGame();
		//do events
		eventSystem.handleEvents();
		//do movements
		objectSystem.moveObjects();
		aimSystem.move();
		//draw objects
		mapSystem.draw(buffer);
//		blockSystem.draw(buffer, mapSystem.getClientX(), mapSystem.getClientY());
		objectSystem.drawObjects(buffer, mapSystem.getClientX(), mapSystem.getClientY());
		//draw information bar
		infoBar.draw(buffer);
		if(robot->isVisible())
			robot->draw(buffer, 0, 0);
		//draw dialog
		dialog.move();
		dialog.draw(buffer, mapSystem.getClientX(), mapSystem.getClientY());
		if(!dialog.isVisible()){
			//draw front sight
			aimSystem.draw(buffer);
		}
		//remove dead ones
		objectSystem.removeDeadObjects();
		//move client
		if (main)
			mapSystem.moveClientTo(main->getCentralX(), main->getCentralY());

		unlockGame();

		//draw FPS
		textprintf_ex(buffer,font,20,0,makecol(255,255,255),-1,
		              "FPS: %d  Objects: %d",fps, objectSystem.getObjectCount());
		if (this->main) {
			textprintf_ex(buffer,font,200,0,makecol(255,255,255),-1,
			              "Kills: %d  Headshots: %d  X: %d  Y: %d", main->getKills(), 
				      main->getHeadshots(), main->getCentralX(), main->getCentralY() );
		}
		//update screen
		screen.flip();
		//rest(restMs);
		vsync();
	}
}

AntiKill::AISystem& AntiKill::Game::getAISystem()
{
	return this->aiSystem;
}

AntiKill::BlockSystem& AntiKill::Game::getBlockSystem()
{
	return this->blockSystem;
}

AntiKill::ClientNetwork& AntiKill::Game::getClientNetwork()
{
	return this->clientNetwork;
}

AntiKill::EventSystem& AntiKill::Game::getEventSystem()
{
	return this->eventSystem;
}

AntiKill::InputSystem& AntiKill::Game::getInputSystem()
{
	return this->inputSystem;
}

AntiKill::Game& AntiKill::Game::getInstance()
{
	return *instance;
}

AntiKill::MapSystem& AntiKill::Game::getMapSystem()
{
	return this->mapSystem;
}

AntiKill::MenuSystem& AntiKill::Game::getMenuSystem()
{
	return this->menuSystem;
}

AntiKill::ObjectSystem& AntiKill::Game::getObjectSystem()
{
	return this->objectSystem;
}

AntiKill::Screen& AntiKill::Game::getScreen()
{
	return this->screen;
}

AntiKill::ServerNetwork& AntiKill::Game::getServerNetwork()
{
	return this->serverNetwork;
}

AntiKill::SoundSystem& AntiKill::Game::getSoundSystem()
{
	return this->soundSystem;
}

AntiKill::AimSystem& AntiKill::Game::getAimSystem()
{
	return this->aimSystem;
}

AntiKill::Player* AntiKill::Game::getMainPlayer()
{
	return this->main;
}

AntiKill::Player* AntiKill::Game::getPlayer(int id)
{
	for (vector<Player*>::iterator it=players.begin(); it!=players.end(); it++) {
		Player* p = *it;
		if (p->getPlayerId()==id)
			return p;
	}
	return 0;
}

AntiKill::Player* AntiKill::Game::setMainPlayer(Player* p)
{
	this->main = p;
}

void AntiKill::Game::addPlayer(Player* p)
{
	objectSystem.addObject(p);
	players.push_back(p);
}

bool AntiKill::Game::loadMap(const char* file)
{
	string name;
	int mapType, mapw, maph, x, y;
	ifstream ifs(file);
	if (ifs.bad())
		return false;
	players.clear();
	dialog.hide();
	objectSystem.cleanupObjects();
	eventSystem.clear();
	this->main = 0;
	ifs >> mapType;
	ifs >> mapw >> maph;
	mapSystem.initialize(mapw, maph);
	blockSystem.initialize(mapw, maph);
	int r, g, b, objCount;
	string objName;
	ifs >> r >> g >> b;
	mapSystem.setBackgroundColor(r, g, b);
	ifs >> objCount;
	for (int j=0; j<objCount; j++ ) {
		int objType, px, py, pw, ph, layer;
		ifs >> objName >> layer >> objType >> x >> y >> px >> py >> pw >> ph;
		switch (objType) {
		case 0: {
			Object* o=new Object();
			o->setX(x).setY(y).setLayer(static_cast<MapLayer>(layer) );
			objName = "image/object/" + objName;
			o->loadImage(objName.c_str());
			o->setCollidable(true);
			o->show();
			objectSystem.addObject(o);
			break;
		}
		case 1: {
			Object* o=new Object();
			o->setX(x).setY(y).setLayer(static_cast<MapLayer>(layer) );
			objName = "image/object/" + objName;
			o->loadImage(objName.c_str());
			o->setCollidable(false);
			o->show();
			objectSystem.addObject(o);
			break;
		}
		}
	}
}

AntiKill::ScriptSystem& AntiKill::Game::getScriptSystem()
{
	return this->scriptSystem;
}

bool AntiKill::Game::isRunning() const
{
	return !this->end;
}
void AntiKill::Game::endGame()
{
	this->end = true;
}

AntiKill::Dialog& AntiKill::Game::getDialog()
{
	return this->dialog;
}

AntiKill::InfoBar& AntiKill::Game::getInfoBar()
{
	return this->infoBar;
}

void* AntiKill::Game::getPlayers()
{
	return &this->players;
}

 AntiKill::Robot& AntiKill::Game::getRobot()
{
	return *this->robot;
}

void AntiKill::Game::lockControl()
{
	this->control = false;
	if(this->main){
		this->main->stopMoving();
	}
}

void AntiKill::Game::unlockControl()
{
	this->control = true;
}

bool AntiKill::Game::canControl() const
{
	return this->control;
}

