#ifndef GAME_H
#define GAME_H

#include "screen.h"
#include "inputsystem.h"
#include "eventsystem.h"
#include "mapsystem.h"
#include "clientnetwork.h"
#include "servernetwork.h"
#include "objectsystem.h"
#include "soundsystem.h"
#include "aisystem.h"
#include "aimsystem.h"
#include "menusystem.h"
#include "blocksystem.h"
#include "scriptsystem.h"
#include "player.h"
#include "dialog.h"
#include "infobar.h"
#include "robot.h"


#include <vector>
using namespace std;

namespace AntiKill
{

class Game {
private:
	Screen		screen;
	InputSystem	inputSystem;
	MapSystem	mapSystem;
	EventSystem	eventSystem;
	ClientNetwork	clientNetwork;
	ServerNetwork	serverNetwork;
	ObjectSystem	objectSystem;
	SoundSystem	soundSystem;
	AISystem	aiSystem;
	MenuSystem	menuSystem;
	BlockSystem	blockSystem;
	AimSystem	aimSystem;
	ScriptSystem	scriptSystem;
	InfoBar		infoBar;
	vector<Player*> players;
	Player*		main;
	bool		end;
	Dialog		dialog;
	Robot*		robot;
	bool		control;
private:
	void gameLoop();
public:

	Game();
	~Game();
	bool start();
	Player*		getMainPlayer();
	Player*		getPlayer(int id);
	Player*		setMainPlayer(Player* p);
	void		addPlayer(Player* p);
	Screen& 	getScreen();
	InputSystem& 	getInputSystem();
	MapSystem& 	getMapSystem();
	EventSystem & 	getEventSystem();
	ClientNetwork& 	getClientNetwork();
	ServerNetwork& 	getServerNetwork();
	ObjectSystem& 	getObjectSystem();
	SoundSystem& 	getSoundSystem();
	ScriptSystem&	getScriptSystem();
	AISystem& 	getAISystem();
	MenuSystem&	getMenuSystem();
	BlockSystem&	getBlockSystem();
	AimSystem&	getAimSystem();
	Dialog&		getDialog();
	InfoBar&	getInfoBar();
	Robot&		getRobot();
	void*		getPlayers();
	static Game& 	getInstance() ;
	bool		loadMap(const char* file);
	bool		isRunning()const;
	void		endGame();
	void		lockControl();
	void		unlockControl();
	bool		canControl ()const;
};

}

#endif // GAME_H
