#include "grenadethrown.h"
#include "game.h"
#include "explosion.h"

namespace AntiKill
{

GrenadeThrown::~GrenadeThrown()
{
}

}

AntiKill::GrenadeThrown::GrenadeThrown(Player* p, int srcX, int srcY, int destX, int destY, int strength)
{
	stickToGround = true;
	Game& g = Game::getInstance();
	float k, dy, dx;
	Vector v;
	setX(srcX);
	setY(srcY);
	this->owner = p;
	this->destX = destX;
	this->destY = destY;
	this->srcX = srcX;
	this->srcY = srcY;
	dy = destY - srcY;
	dx = destX - srcX;
	show();
	speed = 8.0;
	//set up velocity
	if( destY==srcY ){
		v.x = destX > srcX ? speed:-speed;
		v.y = 0.0;
	}else if(destX==srcX){
		v.x = 0.0;
		v.y = destY > srcY ? speed:-speed;
	}else{
		k = dy / dx;
		v.x = sqrtf(speed*speed/(k*k+1));
		if(dx<0)
			v.x = -v.x;
		v.y = k*v.x;
	}
	
	setVelocity(v);
	born = time(0);
	lifetime = 2;
	hitSoundDelay = 0;
}



void AntiKill::GrenadeThrown::draw(BITMAP* dest, int cx, int cy)
{
	int wx = getX()-cx;
	int wy = getY()-cy;
	circlefill(dest, wx, wy, 5, makecol(0,0,0) );
}

void AntiKill::GrenadeThrown::move()
{
	RealBody::move();
	if(hitSoundDelay>0)
		hitSoundDelay --;
	if(time(0)-born >= lifetime){
		Game& g = Game::getInstance();
		Explosion* e = new Explosion(this->owner, this->getX(), this->getY() );
		g.getObjectSystem().addObject(e);
		g.getSoundSystem().playSound("grenade_explode");
		die();
	}
}

void AntiKill::GrenadeThrown::handleCollision(int dir, Object* o)
{
	RealBody::handleCollision(dir, o);
	Player* p = dynamic_cast<Player*>(o);
	if(dir==DIR_BOTTOM){
		if(!p){
			if(hitSoundDelay<=0){
				Game& g = Game::getInstance();
				g.getSoundSystem().playSound("grenade_hitground");
				hitSoundDelay = 5;
			}
		}
	}
//	if(p && p->getColor()!=this->owner->getColor())
//			lifetime = 0;
}

bool AntiKill::GrenadeThrown::collide(Object* o)
{
	Player* p = dynamic_cast<Player*>(o);
	if(p){
		if(p->getColor()==this->owner->getColor())
			return false;
			
	}
	return RealBody::collide(o);
}

void AntiKill::GrenadeThrown::bomb()
{
	lifetime = 0;
}

