#include "infobar.h"
#include "game.h"

namespace AntiKill
{

InfoBar::InfoBar()
{
	bar = 0;
}

InfoBar::~InfoBar()
{
	if(bar)
		destroy_bitmap(bar);
}

}

void AntiKill::InfoBar::draw(BITMAP* dest)
{
	if(visible){
		Game &g = Game::getInstance();
		Player* p = g.getMainPlayer();
		Weapon* w = 0;
		if(p){
			setBlood(p->getHealth());
			setDefend(p->getDefend());
			setEquipment(p->getEquipment());
			w = p->getCurrentWeapon();
			if(w){
				setBullet(w->getBulletCurrent());
				setBackup(w->getBulletTotal()-w->getBulletCurrent());
			}
			
		}
		set_trans_blender(0,0,0,200);
		draw_trans_sprite(dest, back, 0, 0);
		blit(blood, dest, 0, 0, 128, 52, defend->w*bloodValue/100, defend->h);
		blit(defend, dest, 0, 0, 128, 70, defend->w*defendValue/100, defend->h);
		if(w){
			textprintf_ex(dest, font, 392, 30, makecol(255, 255, 255), -1, "%d", backupCount);
			for(int i=0; i<bulletCount; i++){
				draw_sprite(dest, bullet, 392+8*i, 50);
			}
		}
		switch(equipment){
		case EquipPistol:
			draw_sprite(dest, pistol, 275, 15);
			break;
		case EquipGun:
			draw_sprite(dest, gun, 275, 15);
			break;
		case EquipGrenade:
			draw_sprite(dest, grenade, 275, 15);
			break;
		}
	}
}

void AntiKill::InfoBar::initialize()
{
	Game &g = Game::getInstance();
	bloodValue = defendValue = equipment = bulletCount = 0;
	back = load_bitmap("./image/gui/back.bmp", 0);
	defend = load_bitmap("./image/gui/defend.bmp", 0);
	blood = load_bitmap("./image/gui/blood.bmp", 0);
	gun = load_bitmap("./image/gui/gun.bmp", 0);
	pistol = load_bitmap("./image/gui/pistol.bmp", 0);
	grenade = load_bitmap("./image/gui/grenade.bmp", 0);
	bullet = load_bitmap("./image/gui/bullet.bmp", 0);
	if(!back||!defend||!blood||!gun||!pistol||!grenade||!bullet){
		allegro_message("Missing gui picture files.");
		exit(1);
	}
	bar = create_bitmap(g.getMapSystem().getClientWidth(), g.getMapSystem().getClientHeight());
	visible = false;
}

void AntiKill::InfoBar::setBlood(int v)
{
	bloodValue = v;
}

void AntiKill::InfoBar::setBullet(int v)
{
	bulletCount = v;
}

void AntiKill::InfoBar::setDefend(int v)
{
	defendValue = v;
}

void AntiKill::InfoBar::setEquipment(int v)
{
	equipment = v;
}

void AntiKill::InfoBar::refresh(BITMAP* bar)
{
	clear_to_color(bar, makecol(255,0,255));
	blit(back, bar, 0, 0, 0, 0, back->w, back->h);
	blit(blood, bar, 0, 0, 128, 52, defend->w*(bloodValue/100), defend->h);
	blit(defend, bar, 0, 0, 128, 70, defend->w*(defendValue/100), defend->h);
	for(int i=0; i<bulletCount; i++){
		draw_sprite(bar, bullet, 392+25*i, 66);
	}
	switch(equipment){
	case EquipPistol:
		draw_sprite(bar, pistol, 275, 15);
		break;
	case EquipGun:
		draw_sprite(bar, gun, 275, 15);
		break;
	case EquipGrenade:
		draw_sprite(bar, grenade, 275, 15);
		break;
	}
}

void AntiKill::InfoBar::hide()
{
	this->visible = false;
}

void AntiKill::InfoBar::show()
{
	this->visible = true;
}

void AntiKill::InfoBar::refresh()
{
	refresh(this->bar);
}

void AntiKill::InfoBar::setBackup(int v)
{
	this->backupCount = v;
}

