#include "mapsystem.h"
#include "game.h"

namespace AntiKill
{

MapSystem::MapSystem()
{
	clientX = clientY = 0;
	bitmap = 0;
	mapWidth = 640;
	mapHeight = 480;
}

MapSystem::~MapSystem()
{
}

}

void AntiKill::MapSystem::draw(BITMAP* dest)
{
	if( bitmap ){
		blit(bitmap, dest, clientX, clientY, 0, 0, dest->w, dest->h );
	}else{
		clear_to_color(dest, this->backgroundColor);
	}
}

int AntiKill::MapSystem::getClientX() const
{
	return clientX;
}

int AntiKill::MapSystem::getClientY() const
{
	return clientY;
}

AntiKill::MapSystem& AntiKill::MapSystem::setClientX(int x)
{
	this->clientX = x;
	return *this;
}

AntiKill::MapSystem& AntiKill::MapSystem::setClientY(int y)
{
	this->clientY = y;
	return *this;
}

int AntiKill::MapSystem::getHeight() const
{
	return this->mapHeight;
}

int AntiKill::MapSystem::getWidth() const
{
	return this->mapWidth;
}

void AntiKill::MapSystem::initialize(int w, int h)
{
	this->mapWidth = w;
	this->mapHeight = h;
}

AntiKill::Object* AntiKill::MapSystem::getFollowObject()
{
	return this->followObject;
}

void AntiKill::MapSystem::setFollowObject(Object* o)
{
	this->followObject = o;
}

void AntiKill::MapSystem::moveClient()
{
	if(this->followObject)
		moveClientTo(this->followObject->getX(), this->followObject->getY());
}

void AntiKill::MapSystem::moveClientTo(int x, int y)
{
	int dx = x - this->clientX, dy = y - this->clientY;
	if( dx > (this->clientWidth - this->scrollWidth) ){
		this->clientX = x - (this->clientWidth - this->scrollWidth);
	}else if(dx < this->scrollWidth ){
		this->clientX = x - this->scrollWidth;
	}
	if( dy > (this->clientHeight - this->scrollHeight)){
		this->clientY = y - (this->clientHeight - this->scrollHeight);
	}else if( dy < this->scrollHeight){
		this->clientY = y - this->scrollHeight;
	}
	if(this->clientX < 0)
		this->clientX = 0;
	if(this->clientY < 0)
		this->clientY = 0;
	if( this->clientX + this->clientWidth > this->mapWidth)
		this->clientX = this->mapWidth - this->clientWidth;
	if( this->clientY + this->clientHeight > this->mapHeight)
		this->clientY = this->mapHeight - this->clientHeight;
}

AntiKill::MapSystem& AntiKill::MapSystem::setClientHeight(int h)
{
	this->clientHeight = h;
	this->scrollHeight = h / 2;
}

AntiKill::MapSystem& AntiKill::MapSystem::setClientWidth(int w)
{
	this->clientWidth = w;
	this->scrollWidth = w / 2;
}

int AntiKill::MapSystem::getClientHeight() const
{
	return this->clientHeight;
}

int AntiKill::MapSystem::getClientWidth() const
{
	return this->clientWidth;
}

bool AntiKill::MapSystem::inClient(int x, int y)
{
	if(this->clientX<x && x<this->clientX+this->clientWidth &&
		this->clientY<y && y<this->clientY+this->clientHeight)
		return true;
	return false;
}

void AntiKill::MapSystem::setBackgroundColor(int r, int g, int b)
{
	this->backgroundColor = makecol(r,g,b);
}

