#ifndef OBJECT_H
#define OBJECT_H

#include <allegro.h>
#include <cmath>


enum MapLayer{
	BackgroundLayer,
	ObjectLayer,
	EffectLayer
};

struct Rect{
	int x1, x2;
	int y1, y2;
	bool operator==(const Rect& b) const{
		if(x1==b.x1 && x2==b.x2 && y1==b.y1 && y2==b.y2)
			return true;
		else
			return false;
	}
	bool operator!=(const Rect& b) const{
		return !(*this==b);
	}
};

struct Vector{
	float x;
	float y;
	Vector(){
		zero();
	}
	Vector( float mx, float my ){
		x = mx, y = my;
	}
	float getMagnitude(){
		return sqrtf(x*x+y*y);
	}
	void zero(){
		x = y = 0.0;
	}
	void normalize(){
		float m = getMagnitude();
		x /= m;
		y /= m;
	}
	Vector operator*(float m){
		Vector tmp(x*m, y*m);
		return tmp;
	}
	Vector operator+(Vector& v){
		Vector tmp(v.x+x, v.y+y);
		if( fabs(tmp.x) < 0.1)
			tmp.x = 0.0;
		if( fabs(tmp.y) < 0.1)
			tmp.y = 0.0;
		return tmp;
	}
	Vector operator-(Vector& v){
		Vector tmp(v.x-x, v.y-y);
		if( fabs(tmp.x) < 0.1)
			tmp.x = 0.0;
		if( fabs(tmp.y) < 0.1w)
			tmp.y = 0.0;
		return tmp;
	}
	Vector& operator+=(Vector& v){
		return (*this=*this+v);
	}
	Vector& operator*=(float m){
		x*=m; y*=m;
		return *this;
	}
	Vector& operator-=(Vector& v){
		return (*this=*this-v);
	}
	float getDistanceFrom(const Vector& v){
		return sqrt((x-v.x)*(x-v.x)+(y-v.y)*(y-v.y));
	}
};

namespace AntiKill
{

class Object {
protected:
	BITMAP* bitmap;
	int width;
	int height;
	bool visible;
	bool dead;
	int lifetime;
	Vector position;
	Vector velocity;
	Vector acceleration;
	bool collidable;
	Rect rect;
	float maxSpeed;
	float	frictionConstant;
	MapLayer	layer;
	int	id;
private:
	void updateRectangle();
public:
	Object();
	virtual ~Object();
	Object& setX(int x);
	Object& setY( int y);
	int getX() const;
	int getY() const;
	Vector getVelocity() const;
	Vector getPosition() const;
	Vector getAcceleration() const;
	float getFrictionConstant() const;
	virtual void draw( BITMAP* dest, int x, int y );
	virtual void move();
	virtual void stop();
	virtual void die();
	virtual bool collide(Object* );
	virtual void handleCollision(int dir, Object* o);
	bool isCollidable() const;
	void show();
	void hide();
	bool isVisible() const;
	int getLifetime() const;
	bool isDead() const;
	int getHeight() const;
	int getWidth() const;
	int getCentralX() const;
	int getCentralY() const;
	MapLayer getLayer() const;
	int getId() const;
	const Rect& getRectangle();
	Object& setLayer(MapLayer l);
	Object& setWidth(int w);
	Object& setHeight(int h);
	Object& setLifetime(int t);
	Object& setVelocity(Vector& v);
	Object& setPosition(Vector& p);
	Object& setAcceleration(Vector& a);
	Object& setCollidable(bool c);
	bool loadImage(const char* file);
};

}

#endif // OBJECT_H
