#include "objectsystem.h"
#include "game.h"

namespace AntiKill
{

ObjectSystem::ObjectSystem()
{
	objects.clear();
	temp.empty();
	iterating = false;
}

ObjectSystem::~ObjectSystem()
{
}

}

void AntiKill::ObjectSystem::addObject(Object* o)
{
	Game& g = Game::getInstance();
	if(iterating){
		temp.push(o);
	}else{
		objects.push_back(o);
		if(o->isCollidable())
			g.getBlockSystem().addObject(o);
	}
}

void AntiKill::ObjectSystem::cleanupObjects()
{
	for( vector<Object*>::iterator it=objects.begin(); it!=objects.end(); it++ ){
		Object* o = *it;
		delete o;
	}
	objects.clear();
}

void AntiKill::ObjectSystem::drawObjects(BITMAP* dest, int x, int y)
{
	iterating = true;
	for( vector<Object*>::iterator it=objects.begin(); it!=objects.end(); it++ ){
		Object* o = *it;
		if( o->isVisible() ){
			o->draw(dest, x, y);
		}
	}
	iterating = false;
}

void AntiKill::ObjectSystem::moveObjects()
{
	Game& g = Game::getInstance();
	iterating = true;
	for( vector<Object*>::iterator it=objects.begin(); it!=objects.end(); it++ ){
		Object* o = *it;
		if( !o->isDead() ){
			const Rect r1 = o->getRectangle();
			o->move();
			const Rect& r2 = o->getRectangle();
			if(o->isCollidable()&& r1!=r2 ){
				g.getBlockSystem().removeObject(o, r1.x1, r1.y1, r1.x2, r1.y2);
				if(g.getBlockSystem().checkCollision(o)){
				//	o->stop();
				//	o->setX(oldx);
				//	o->setY(oldy);
				}else{
					Player* p = dynamic_cast<Player*>(o);
					if(p){
						p->setFree(true);
						p->setBoard(0);
					}
					RealBody* r = dynamic_cast<RealBody*>(o);
					if(r)
						r->releaseRoad();
				}
				g.getBlockSystem().addObject(o);
			}
		}
	}
	iterating = false;
}

bool AntiKill::ObjectSystem::removeObject(Object* o)
{
	if(iterating){
		allegro_message("You should not remove Object when iterating ..");
		return false;
	}
	for( vector<Object*>::iterator it=objects.begin(); it!=objects.end(); it++ ){
		if( o==*it ){
			objects.erase(it);
			return true;
		}
	}
	return false;
}

void AntiKill::ObjectSystem::removeDeadObjects()
{
	Game& g = Game::getInstance();
	iterating = true;
	for( vector<Object*>::iterator it=objects.begin(); it!=objects.end(); ){
		Object* o = *it;
		if(o->isDead()){
			objects.erase(it);
			if(o->isCollidable())
				g.getBlockSystem().removeObject(o);
			delete o;
		}else{
			it ++;
		}
	}
	iterating = false;
	// add new ones
	while(temp.size()>0){
		Object* ob = temp.front();
		addObject(ob);
		temp.pop();
	}
}

int AntiKill::ObjectSystem::getObjectCount() const
{
	return objects.size();
}

AntiKill::Object* AntiKill::ObjectSystem::getObjectById(int id)
{
	for( vector<Object*>::iterator it=objects.begin(); it!=objects.end(); it++ ){
		Object* o = *it;
		if(o->getId()==id)
			return o;
	}
	return 0;
}

