#ifndef PLAYER_H
#define PLAYER_H

#include "realbody.h" // Base class: AntiKill::RealBody
#include "animation.h"
#include "weapon.h"
#include <iostream>

enum PlayerState{
	NormalState =0,
	WalkingState,
	RunningState,
	JumpingState,
	ShootingState,
	SquatingState,
	DyingState,
	DeadState
};

namespace AntiKill
{

enum ActionType{
	ActionMove = 1,
	ActionFire,
	ActionJump
};

struct PlayerAction{
	ActionType	type;
	int		x;
	int		y;
};

class Player : public AntiKill::RealBody {
protected:
	string	playerName;
	int	playerId;
	int	health;
	int	defend;
	int	headshots;
	int	kills;
	bool	acting;
	int 	stepSize;
	int	color;
	int	equipment;
	string	role;
	Animation animation;
	enum PlayerState state;
	bool	movingLeft, movingRight, firing, jumping;
	int	fireX, fireY;
	int	aimX, aimY;
	PlayerAction	playerAction;
	bool	playerActionOn;
	Weapon*	weapons[4];
private:
	void initialize();
public:
	Player(string name, string role);
	Player();
	virtual ~Player();
	void	setName(string name);
	string	getName()const;
	int	getHealth()const;
	void	setHealth(int h);
	void	setDefend(int d);
	int	getPlayerId()const;
	int	getDefend()const;
	string	getRole()const;
	bool	isAlive() const;
	int	getKills()const;
	int	getEquipment()const;
	int	getHeadshots()const;
	Weapon*	getCurrentWeapon();
	Weapon* getWeapon(int e);
	void	setWeapon(int e, Weapon*w);
	void	setEquipment(int v);
	void	addKills();
	void	addHeadshots();
	int	getColor() const;
	void	setColor(int i);
	void	setRole(string role);
	bool	hit(int x, int y, Vector& dir);	
	void	aim(int x, int y);
	void	moveLeft(bool t);
	void	moveRight(bool t);
	void	stopMoving();
	void	fire(int x, int y);
	bool	isFiring()const;
	void	jump(bool t);
	void	act(enum PlayerState st);
	virtual void die();
	virtual void draw(BITMAP* dest, int x, int y );
	virtual void move();
	virtual bool collide(Object*);
	void	holdFire();
	void	stopAction();
	void	setAction(PlayerAction &a);
	bool	isPlayerActionOn() const;
	virtual void doFire();
	void	doJump();
	virtual	void doMoveLeft();
	virtual	void doMoveRight();
	void	doPlayerAction();
	void	setWeapon(string type);
	
};

}

#endif // PLAYER_H
