#include "realbody.h"
#include "animation.h"
#include <iostream>

namespace AntiKill
{

const	float gravityAccel = 0.12;

RealBody::RealBody()
{
	mass = 1;
	free = true;
	collidable = true;
	on = 0;
	stickToGround = false;
	load.clear();
}

RealBody::~RealBody()
{
}

}

bool AntiKill::RealBody::collide(Object* o)
{
	bool collision = Object::collide(o);
	return collision;
}

void AntiKill::RealBody::die()
{
	Object::die();
}

void AntiKill::RealBody::draw(BITMAP* dest, int x, int y)
{
	Object::draw(dest, x, y);
}

void AntiKill::RealBody::move()
{
	Object::move();
	 //Gravity
	if( this->free && on==0 ){
		this->acceleration = Vector(0.0, gravityAccel);
	}
	if( !this->free && on ){
		if(this->velocity.x < 0 ){
			this->velocity.x += min(abs(this->velocity.x), this->frictionAccel );
		}else if(this->velocity.x > 0 ){
			this->velocity.x -= min(abs(this->velocity.x), this->frictionAccel );
		}
	}
}

void AntiKill::RealBody::stop()
{
	Object::stop();
}

int AntiKill::RealBody::getMass() const
{
	return this->mass;
}

AntiKill::RealBody& AntiKill::RealBody::setMass(int m)
{
	this->mass = m;
	return *this;
}

void AntiKill::RealBody::addForce(Vector& v)
{
	Vector a( v.x/mass, v.y/mass );
	this->acceleration += a;
}

void AntiKill::RealBody::addMomentum(Vector& p)
{
	Vector v( p.x/mass, p.y/mass );
	this->velocity += v;
}


bool AntiKill::RealBody::isFree() const
{
	return this->free;
}

AntiKill::RealBody& AntiKill::RealBody::setFree(bool f)
{
	this->free = f;
	if(f)
		this->on = 0;
	return *this;
}

void AntiKill::RealBody::handleCollision(int dir, Object* o)
{
	RealBody* r = dynamic_cast<RealBody*>(o);
	switch(dir){
	case DIR_TOP:
		this->velocity.y = -this->velocity.y;
		break;
	case DIR_BOTTOM: //onboard ?
		if(stickToGround && fabsf(this->velocity.y)>1.0){
			this->velocity.y = -(this->velocity.y + o->getVelocity().y) / 2;
			break;
		}
		this->velocity.y = 0.0;
		this->on = o;
		if(r)
			r->load.push_back(this);
		this->acceleration = Vector(0.0,0.0);
		this->free = false;
		this->frictionAccel = o->getFrictionConstant() * gravityAccel;
		break;
	case DIR_LEFT:
	case DIR_RIGHT:
//		this->velocity.x = -this->velocity.x*(1.0-o->getFrictionConstant());
		this->velocity.x = -(this->velocity.x + o->getVelocity().x) / 2;
		break;
	}
}

AntiKill::Object* AntiKill::RealBody::getBoard()
{
	return on;
}

void AntiKill::RealBody::setBoard(Object* o)
{
	this->on = o;
}


void AntiKill::RealBody::releaseRoad()
{
	for(vector<RealBody*>::iterator it=load.begin(); it!=load.end(); it++){
		RealBody* r = *it;
		r->setFree(true);
	}
	load.clear();
}

