#include <allegro.h>
#include <iostream>
#include "screen.h"
using namespace std;

//#define USE_DOUBLE_BUFFERING

namespace AntiKill
{

Screen::Screen()
{
	vbuffer[0]=vbuffer[1]=mbuffer=0;
	index = 0;
	rate = 5;
	fadeInOut = Invisible;
}

Screen::~Screen()
{
}

}

int AntiKill::Screen::getBitsPerPixel() const
{
	return bpp;
}

int AntiKill::Screen::getHeight() const
{
	return height;
}

int AntiKill::Screen::getWidth() const
{
	return width;
}

bool AntiKill::Screen::setVideoMode(int w, int h, int bpp)
{
	set_color_depth(desktop_color_depth());
	if(set_gfx_mode(0, w, h, 0, 0)!=0){
		cout << "Failed to set video mode " << w << "*" << h << "*" << bpp << endl;
		return false;
	}
	this->width = w;
	this->height = h;
	this->bpp = bpp;
	this->index = 0;
	if( vbuffer[0] )
		destroy_bitmap(vbuffer[0]);
	if( vbuffer[1] )
		destroy_bitmap(vbuffer[1]);
	if( mbuffer )
		destroy_bitmap(mbuffer);
	vbuffer[0] = create_video_bitmap( w, h);
	vbuffer[1] = create_video_bitmap( w, h);
	mbuffer = create_bitmap( w, h);
	if( !vbuffer[0] || !vbuffer[1] ){
		cout << "Failed to create video bitmap.";
		return false;
	}
	show_video_bitmap(vbuffer[1]);
	return true;
}


void AntiKill::Screen::flip()
{
	BITMAP* tmp;
	switch(fadeInOut){
	case Invisible:
		break;
	case Visible:
#ifdef USE_DOUBLE_BUFFERING
		show_video_bitmap(vbuffer[index]);
		index = 1-index;
#else
		blit(mbuffer, vbuffer[1-index], 0, 0, 0, 0, mbuffer->w, mbuffer->h);
#endif
		break;
	case FadeIn:
		tmp = create_bitmap(mbuffer->w, mbuffer->h);
		set_trans_blender(0, 0, 0, alpha);
		draw_trans_sprite(mbuffer, tmp, 0, 0);
		blit(mbuffer, vbuffer[1-index], 0, 0, 0, 0, mbuffer->w, mbuffer->h);
		alpha -= rate;
		if(alpha<0)
			fadeInOut = Visible;
		destroy_bitmap(tmp);
		break;
	case FadeOut:
		tmp = create_bitmap(mbuffer->w, mbuffer->h);
		set_trans_blender(0, 0, 0, alpha);
		draw_trans_sprite(mbuffer, tmp, 0, 0);
		blit(mbuffer, vbuffer[1-index], 0, 0, 0, 0, mbuffer->w, mbuffer->h);
		alpha += rate;
		if(alpha>255)
			fadeInOut = Invisible;
		destroy_bitmap(tmp);
		break;
	}
}

BITMAP* AntiKill::Screen::getBufferPage()
{
#ifdef USE_DOUBLE_BUFFERING
	if(fadeInOut==FadeIn||fadeInOut==FadeOut)
		return mbuffer;
	return vbuffer[index];
#else
	return mbuffer;
#endif
}

BITMAP* AntiKill::Screen::getCurrentPage()
{
	return vbuffer[1-index];
}

void AntiKill::Screen::fadeIn()
{
	alpha = 255;
	fadeInOut = FadeIn;
}

void AntiKill::Screen::fadeOut()
{
	alpha = 0;
	fadeInOut = FadeOut;
}

AntiKill::FadeInOut AntiKill::Screen::getFadeMode() const
{
	return this->fadeInOut;
}

