
#include "scriptsystem.h"
#include "lock.h"
#include "game.h"
#include "gun.h"
#include "aiplayer.h"
#include "grenade.h"
#include "pistol.h"
#include "presentbox.h"

namespace AntiKill
{

Game* game;

ScriptSystem::ScriptSystem()
{
	scriptThread = 0;
}

ScriptSystem::~ScriptSystem()
{
}

}

bool AntiKill::ScriptSystem::raise(string event, string name, int x, int y)
{
	ScriptEvent* e = new ScriptEvent;
	e->event = event;
	e->name = name;
	e->x = x;
	e->y = y;
	scriptEvents.push(e);
	return true;
}

void AntiKill::ScriptSystem::runScript()
{
	cout << "Script thread running " << endl;
	if(!script.run("Main")){
		cout << "Runtime error: " << script.getLastError() << endl;
	}
	while( game->isRunning() ){
		while(scriptEvents.size()>0){
			ScriptEvent* e = scriptEvents.front();
			ShrimpVariable id, x, y;
			vector<ShrimpVariable>& runtime = script.getRuntimeStack();
			id.setInteger(e->id);
			x.setInteger(e->x);
			y.setInteger(e->y);
			runtime.push_back(id);
			runtime.push_back(x);
			runtime.push_back(y);
			try{
				script.toUpperString(e->event);
				script.invokeFunction(e->event, 3);
			}catch(ShrimpRuntimeException excep){
				cout << excep.getFunction() << "("<<excep.getLine()<<"): " << excep.getError() 
					<< " : " << excep.getDetail()  << endl;
			}
			runtime.pop_back();
			runtime.pop_back();
			runtime.pop_back();
			delete e;
			scriptEvents.pop();
		}
		rest(100);
	}
	cout << "Script thread ended" << endl;
}

namespace AntiKill{
	static void * threadProcedure(void* data){
		ScriptSystem* s = (ScriptSystem*)data;
		s->runScript();
		return 0;
	}
	
	void print(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		for (int i=pcount; i>0; i--) {
			ShrimpVariable& v = runtimeStack[runtimeStack.size()-i];
			cout << v.getString();
		}
	}

	void command(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		ShrimpVariable& v = runtimeStack.back();
		system(v.getString().c_str());
	}

	void message(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		string msg;
		for (int i=pcount; i>0; i--) {
			ShrimpVariable& v = runtimeStack[runtimeStack.size()-i];
			msg += v.getString();
		}
		allegro_message( msg.c_str() );
	}

	void isNull(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		for (int i=pcount; i>0; i--) {
			ShrimpVariable& v = runtimeStack[runtimeStack.size()-i];
			if (v.type!=0) {
				result.setInteger(0);
				return;
			}
		}
		result.setInteger(1);
	}

	void loadMap(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		ShrimpVariable& v = runtimeStack.back();
		lockGame();
		game->loadMap(v.getString().c_str());
		unlockGame();
	}

	void addPlayer(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int y = runtimeStack[runtimeStack.size()-1].getInteger();
		int x = runtimeStack[runtimeStack.size()-2].getInteger();
		string name = runtimeStack[runtimeStack.size()-3].getString();
		string role = runtimeStack[runtimeStack.size()-4].getString();
		int team = runtimeStack[runtimeStack.size()-5].getInteger();
		int type = runtimeStack[runtimeStack.size()-6].getInteger();
		Player* p;
		lockGame();
		if(type==0){ //Add main player
			p = new Player(name, role);
			p->setX(x).setY(y);
			p->setColor(team);
			game->addPlayer(p);
			game->setMainPlayer(p);
		}else{
			p = new AIPlayer(name, role);
			p->setX(x).setY(y);
			p->setColor(team);
			game->addPlayer(p);
		}
		result.setInteger(p->getId());
		unlockGame();
	}
	
	void addPresent(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int y = runtimeStack[runtimeStack.size()-1].getInteger();
		int x = runtimeStack[runtimeStack.size()-2].getInteger();
		string pic = runtimeStack[runtimeStack.size()-3].getString();
		lockGame();
		PresentBox *p = new PresentBox();
		p->setX(x).setY(y);
		string file = "image/object/" + pic + ".bmp";
		p->loadImage(file.c_str());
		p->show();
		game->getObjectSystem().addObject(p);
		result.setInteger(p->getId());
		unlockGame();
	}
	
	void addHiddenPresent(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int h = runtimeStack[runtimeStack.size()-1].getInteger();
		int w = runtimeStack[runtimeStack.size()-2].getInteger();
		int y = runtimeStack[runtimeStack.size()-3].getInteger();
		int x = runtimeStack[runtimeStack.size()-4].getInteger();
		lockGame();
		PresentBox *p = new PresentBox();
		p->setX(x).setY(y);
		p->setWidth(w).setHeight(h);
		p->show();
		cout << "add hidden present " << x << ", " << y << ", " << w << ", " << h << endl;
		game->getObjectSystem().addObject(p);
		result.setInteger(p->getId());
		unlockGame();
	}
	
	void showObject(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		ShrimpVariable& v = runtimeStack.back();
		lockGame();
		Object* o = game->getObjectSystem().getObjectById(v.getInteger());
		if(o)
			o->show();
		else
			cout << "Object not found " << v.getInteger() << endl;
		unlockGame();
	}
	
	void setPlayerHealth(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int h = runtimeStack[runtimeStack.size()-1].getInteger();
		int id = runtimeStack[runtimeStack.size()-2].getInteger();
		lockGame();
		Object* o = game->getObjectSystem().getObjectById(id);
		Player* p = dynamic_cast<Player*>(o);
		if(p)
			p->setHealth(h);
		unlockGame();
	}
	
	void fadeIn(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		game->getScreen().fadeIn();
		while(game->getScreen().getFadeMode()==FadeIn)
			rest(100);
	}
	
	void fadeOut(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		game->getScreen().fadeOut();
		while(game->getScreen().getFadeMode()==FadeOut)
			rest(100);
	}
	
	void delay(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int t = runtimeStack.back().getInteger();
		rest(t);
	}
	
	void endgame(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		game->endGame();
	}
	
	void addPlayerWeapon(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int bulletCount = runtimeStack[runtimeStack.size()-1].getInteger();
		int weapon = runtimeStack[runtimeStack.size()-2].getInteger();
		int id = runtimeStack[runtimeStack.size()-3].getInteger();
		lockGame();
		Object* o = game->getObjectSystem().getObjectById(id);
		Player* p = dynamic_cast<Player*>(o);
		if(p){
			Weapon* w = p->getWeapon(weapon);
			if(!w){
				switch(weapon){
				case EquipGun:
					w = new Gun(p);
					break;
				case EquipPistol:
					w = new Pistol(p);
					break;
				case EquipGrenade:
					w = new Grenade(p);
					break;
				}
				p->setWeapon(weapon, w);
			}
			if(w){
				w->addBullets(bulletCount);
			}
		}
		unlockGame();
	}
	
	void switchWeapon(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int weapon = runtimeStack[runtimeStack.size()-1].getInteger();
		int id = runtimeStack[runtimeStack.size()-2].getInteger();
		lockGame();
		Object* o = game->getObjectSystem().getObjectById(id);
		Player* p = dynamic_cast<Player*>(o);
		if(p){
			p->setEquipment(weapon);
		}
		unlockGame();
	}
	
	void playerDoAim(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int x = runtimeStack[runtimeStack.size()-1].getInteger();
		int y = runtimeStack[runtimeStack.size()-2].getInteger();
		int id = runtimeStack[runtimeStack.size()-3].getInteger();
		lockGame();
		Object* o = game->getObjectSystem().getObjectById(id);
		Player* p = dynamic_cast<Player*>(o);
		if(p){
			p->aim(x, y);
		}
		unlockGame();
	}
	void playerDoFire(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int y = runtimeStack[runtimeStack.size()-1].getInteger();
		int x = runtimeStack[runtimeStack.size()-2].getInteger();
		int id = runtimeStack[runtimeStack.size()-3].getInteger();
		lockGame();
		Object* o = game->getObjectSystem().getObjectById(id);
		Player* p = dynamic_cast<Player*>(o);
		if(p){
			Weapon* w = p->getCurrentWeapon();
			if(w)
				w->fire(x, y);
		}
		unlockGame();
	}
	void playerDoMove(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int y = runtimeStack[runtimeStack.size()-1].getInteger();
		int x = runtimeStack[runtimeStack.size()-2].getInteger();
		int id = runtimeStack[runtimeStack.size()-3].getInteger();
		lockGame();
		Object* o = game->getObjectSystem().getObjectById(id);
		Player* p = dynamic_cast<Player*>(o);
		if(p){
			PlayerAction pa;
			pa.type = ActionMove;
			pa.x = x;
			pa.y = y;
			p->setAction(pa);
		}
		unlockGame();
	}
	void playerDoStop(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int id = runtimeStack[runtimeStack.size()-1].getInteger();
		lockGame();
		Object* o = game->getObjectSystem().getObjectById(id);
		Player* p = dynamic_cast<Player*>(o);
		if(p){
			p->stopAction();
			p->stopMoving();
			p->stop();
		}
		unlockGame();
	}
	void say(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		string text = runtimeStack[runtimeStack.size()-1].getString();
		int id = runtimeStack[runtimeStack.size()-2].getInteger();
		Object* o = game->getObjectSystem().getObjectById(id);
		Player* p = dynamic_cast<Player*>(o);
		if(p){
			game->getDialog().hide().setPosition(p->getCentralX(), 
				p->getY()).setText(text).setColor(p->getColor()).show();
			while(game->getDialog().isVisible())
				rest(100);
		}
	}
	void robotSay(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		string text = runtimeStack[runtimeStack.size()-1].getString();
		Robot& robot = game->getRobot();
		game->getDialog().hide().setPosition(robot.getX()+robot.getWidth()+game->getMapSystem().getClientX(), 
			robot.getCentralY()+game->getMapSystem().getClientY()).setText(text)
			.setColor(makecol(255,0,0)).show();
		while(game->getDialog().isVisible()){
			rest(100);
		}
	}
	void showRobot(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int show = runtimeStack[runtimeStack.size()-1].getInteger();
		Robot& robot = game->getRobot();
		if(show)
			robot.show();
		else
			robot.hide();
	}
	void lockControl(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int c = runtimeStack[runtimeStack.size()-1].getInteger();
		if(c)
			game->lockControl();
		else
			game->unlockControl();
	}
	void waitForPlayerAction(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int id = runtimeStack[runtimeStack.size()-1].getInteger();
		for(;;){
			lockGame();
			Object* o = game->getObjectSystem().getObjectById(id);
			Player* p = dynamic_cast<Player*>(o);
			if(p){
				if(!p->isPlayerActionOn()){
					unlockGame();
					break;
				}
			}
			unlockGame();
			rest(100);
		}
	}
	void setPlayerAIMode(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int b = runtimeStack[runtimeStack.size()-1].getInteger();
		int id = runtimeStack[runtimeStack.size()-2].getInteger();
		lockGame();
		Object* o = game->getObjectSystem().getObjectById(id);
		AIPlayer* p = dynamic_cast<AIPlayer*>(o);
		if(p){
			p->setControlMode(b);
		}
		unlockGame();
	}
	void die(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int id = runtimeStack[runtimeStack.size()-1].getInteger();
		lockGame();
		Object* o = game->getObjectSystem().getObjectById(id);
		Player* p = dynamic_cast<Player*>(o);
		if(p){
			p->die();
		}
		unlockGame();
	}
	void playerX(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int id = runtimeStack[runtimeStack.size()-1].getInteger();
		lockGame();
		Object* o = game->getObjectSystem().getObjectById(id);
		Player* p = dynamic_cast<Player*>(o);
		if(p){
			result.setInteger(p->getCentralX());
		}
		unlockGame();
	}
	void playerY(vector<ShrimpVariable>& runtimeStack, ShrimpVariable& result, int pcount )
	{
		int id = runtimeStack[runtimeStack.size()-1].getInteger();
		lockGame();
		Object* o = game->getObjectSystem().getObjectById(id);
		Player* p = dynamic_cast<Player*>(o);
		if(p){
			result.setInteger(p->getCentralY());
		}
		unlockGame();
	}
}

bool AntiKill::ScriptSystem::start()
{
	script.addFunction("print", &print);
	script.addFunction("isNull", &isNull);
	script.addFunction("message", &message);
	script.addFunction("system", &command);
	script.addFunction("loadMap", &loadMap);
	script.addFunction("addPlayer", &addPlayer);
	script.addFunction("showObject", &showObject);
	script.addFunction("setPlayerHealth", &setPlayerHealth);
	script.addFunction("fadeIn", &fadeIn);
	script.addFunction("fadeOut", &fadeOut);
	script.addFunction("delay", &delay);
	script.addFunction("endgame", &endgame);
	script.addFunction("addPlayerWeapon", &addPlayerWeapon);
	script.addFunction("switchWeapon", &switchWeapon);
	script.addFunction("playerDoAim", &playerDoAim);
	script.addFunction("playerDoFire", &playerDoFire);
	script.addFunction("playerDoMove", &playerDoMove);
	script.addFunction("playerDoStop", &playerDoStop);
	script.addFunction("say", &say);
	script.addFunction("robotSay", &robotSay);
	script.addFunction("waitForPlayerAction", &waitForPlayerAction);
	script.addFunction("showRobot", &showRobot);
	script.addFunction("lockControl", &lockControl);
	script.addFunction("addPresent", &addPresent);
	script.addFunction("addHiddenPresent", &addHiddenPresent);
	script.addFunction("setPlayerAIMode", &setPlayerAIMode);
	script.addFunction("die", &die);
	script.addFunction("playerX", &playerX);
	script.addFunction("playerY", &playerY);
	if(!script.load("./script/main.fb"))
		cout << "Script error: " << script.getLastError() << endl;
	game = &Game::getInstance();
	scriptThread = createThread(this, threadProcedure);
}

void AntiKill::ScriptSystem::stop()
{
	if(scriptThread)
		joinThread(scriptThread);
}

bool AntiKill::ScriptSystem::raise(string event, int id, int x, int y)
{
	ScriptEvent* e = new ScriptEvent;
	e->event = event;
	e->id = id;
	e->x = x;
	e->y = y;
	scriptEvents.push(e);
	return true;
}

