/*
 *  Xiaoxia
 *  2010=05=10
 *  gdxxhg@gmail.com
 *  http://home.xxsyzx.com
 *
 */

#include "shrimpscript.h"
#include <cstdio>
#include <cstring>
#include <iostream>

enum {
	TokenSymbol = 0,
	TokenString,			//"123"
	TokenNumber,			//123
	TokenEndOfLine,			//\n
	TokenIF,			//IF
	TokenFOR,			//FOR
	TokenNEXT,			//NEXTf
	TokenELSE,			//ELSE
	TokenEXIT,			//EXIT
	TokenDIM,			//DIM
	TokenIMPORT,			//IMPORT
	TokenCONST,			//CONST
	TokenOR,			//OR
	TokenAND,			//AND
	TokenXOR,			//XOR
	TokenNOT,			//NOT
	TokenMOD,			//MOD
	TokenTO,			//TO
	TokenCALL,			//CALL
	TokenLET,			//LET
	TokenFUNCTION,			//FUNCTION
	TokenEND,			//END
	TokenTHEN,			//THEN
	TokenSTEP,			//STEP
	TokenDO,			//DO
	TokenLOOP,			//LOOP
	TokenUNTIL,			//UNTIL
	TokenWHILE,			//WHILE
	TokenWEND,			//WEND
	TokenSELECT,			//SELECT
	TokenCASE,			//CASE
	TokenComma,			//,
	TokenOperator, 			//()[]=><+-*/
	TokenLeftRoundBracket,		//(
	TokenRightRoundBracket,		//)
	TokenLeftSquareBracket,		//[
	TokenRightSquareBracket,	//]
	TokenEqual2,			//==
	TokenLarger,			//>
	TokenLargerOrEqual,		//>=
	TokenLess,			//<
	TokenLessOrEqual,		//<=
	TokenNotEqual,			//<>
	TokenColon,			//:
	TokenSemicolon,			//;
	TokenEqual,			//=
	TokenAssign,			//:=
	TokenMul,			//*
	TokenDiv,			///
	TokenAdd,			//+
	TokenSub,			//-
	TokenEnd
};

struct OperatorPriority {
	char	op[10];
	int	pri;
	int	opCode;
}opPriority [] = {
	{ ")", 0, TokenRightRoundBracket },
	{ "NOT", 2, TokenNOT },
	{ "*", 3, TokenMul }, { "/", 3, TokenDiv }, { "MOD", 3, TokenMOD },
	{ "+", 4, TokenAdd }, { "-", 4, TokenSub },
	{ "<", 6, TokenLess }, { "<=", 6, TokenLessOrEqual }, { ">", 6, TokenLarger }, { ">=", 6, TokenLargerOrEqual },
	{ "=", 7, TokenEqual }, { "<>", 7, TokenNotEqual },
	{ "AND", 8, TokenAND },
	{ "OR", 9, TokenOR }, { "XOR", 9, TokenXOR },
	{ ":=", 11, TokenAssign },
	{ ",", 12, TokenComma },
	{ "(", 99, TokenLeftRoundBracket }
};

struct TokenString {
	int	token;
	char	value[10];
}tokenStrings[] = {
	{ TokenEndOfLine, "\n" },
	{ TokenIF, "IF" },
	{ TokenFOR, "FOR" },
	{ TokenNEXT, "NEXT" },
	{ TokenELSE, "ELSE" },
	{ TokenEXIT, "EXIT" },
	{ TokenDIM, "DIM" },
	{ TokenIMPORT, "IMPORT" },
	{ TokenCONST, "CONST" },
	{ TokenOR, "OR" },
	{ TokenAND, "AND" },
	{ TokenXOR, "XOR" },
	{ TokenNOT, "NOT" },
	{ TokenMOD, "MOD" },
	{ TokenTO, "TO" },
	{ TokenCALL, "CALL" },
	{ TokenLET, "LET" },
	{ TokenFUNCTION, "FUNCTION" },
	{ TokenEND, "END" },
	{ TokenTHEN, "THEN" },
	{ TokenSTEP, "STEP" },
	{ TokenWHILE, "WHILE" },
	{ TokenDO, "DO" },
	{ TokenUNTIL, "UNTIL" },
	{ TokenLOOP, "LOOP" },
	{ TokenSELECT, "SELECT" },
	{ TokenCASE, "CASE" },
	{ TokenWEND, "WEND" },
	{ TokenComma, "," },
	{ TokenSemicolon, ";" },
	{ TokenColon, ":" }
};

const char* operators = "()[]=><+-*/";

ShrimpScript::ShrimpScript()
{
	setLastError("No error.", "");
}

ShrimpScript::~ShrimpScript()
{
}

string ShrimpScript::getLastError() const
{
	return lastError;
}

bool ShrimpScript::load(string file)
{
	char *buf;
	int len;
	FILE* fp = fopen(file.c_str(), "rb");
	if (!fp) {
		setLastError("Failed to open file.", file);
		return false;
	}
	fseek(fp, 0, SEEK_END);
	len = ftell(fp);
	fseek(fp, 0, SEEK_SET);
	buf = new char[len+1];
	fread(buf, len, 1, fp);
	buf[len] = '\0';
	fclose(fp);
	bool ret = parse(buf);
	delete []buf;
	return ret;
}

bool ShrimpScript::run(string name)
{
	toUpperString(name);
	runtimeStack.empty();
	try {
		invokeFunction(name, 0);
	} catch (ShrimpRuntimeException e) {
		setLastError(e.getFunction()+": "+e.getError(), e.getDetail(), e.getLine());
		return false;
	}
	return true;
}

void ShrimpScript::setLastError(string err, string detail, int line)
{
	char tmp[10];
	itoa(line, tmp, 10);
	if (line == -1)
		lastError = err + " : " + detail;
	else
		lastError = string("At line ") + tmp + " : " + err + " : " + detail;
}

bool ShrimpScript::parse(string code)
{
	enum ParseState {
		ParseWord,
		ParseComment,
		ParseString
	};
	int line = 1;
	ParseState state = ParseWord;
	string	token = "";
	queue<ShrimpToken>* oldSet;
	oldSet = currentTokenSet;
	currentTokenSet = new queue<ShrimpToken>();
	for (int i=0; i<code.length(); i++) {
		char ch = code[i];
		switch (state) {
		case ParseWord:
			switch (ch) {
			case '\r':
				if (code[i+1]=='\n')
					i++;
			case '\n':
				addToken(line, token);
				addToken(line, "\n");
				token = "";
				line ++;
				break;
			case '\'':
				addToken(line, token);
				state = ParseComment;
				token = "";
				break;
			case ' ':
			case '\t':
				addToken(line, token);
				token = "";
				break;
			case '"':
				addToken(line, token);
				token = "\"";
				state = ParseString;
				break;
			default:
				if (isalpha(ch)||isdigit(ch)||ch=='.')
					token += ch;
				else {
					addToken(line, token);
					token = ch;
					addToken(line, token);
					token = "";
				}
			}
			break;
		case ParseComment:
			switch (ch) {
			case '\r':
				if (code[i+1]=='\n')
					i++;
			case '\n':
				addToken(line, "\n");
				line ++;
				state = ParseWord;
				break;
			default:
				break;
			}
			break;
		case ParseString:
			switch (ch) {
			case '"':
				token += ch;
				addToken(line, token);
				state = ParseWord;
				token = "";
				break;
			default:
				token += ch;
			}
		}
	}
	if (state == ParseString ) {
		setLastError("Unclosed string.", "missing \"", line);
		return false;
	}
	bool result = parseSyntax();
	delete currentTokenSet;
	currentTokenSet = oldSet;
	return result;
}

void ShrimpScript::addToken(int line, string value)
{
	if (value.size()==0)
		return;
	ShrimpToken token;
	token.line = line;
	token.value = value;
	token.type = -1;
	if ( value.size()==1 && strchr(operators, value[0]) ) {
		token.type = TokenOperator;
		int last = currentTokenSet->size()-1;
		if (last>=0 && currentTokenSet->back().value.size()==1) {
			switch (currentTokenSet->back().value[0]) {
			case '>':
				if (value[0]=='=') {
					currentTokenSet->back().value = ">=";
					return;
				}
				break;
			case '<':
				if (value[0]=='=') {
					currentTokenSet->back().value = "<=";
					return;
				} else if (value[0]=='>') {
					currentTokenSet->back().value = "<>";
					return;
				}
				break;
			}
		}
	} else if (value[0]=='"') {
		token.type = TokenString;
		int j=0;
		char* tmp = new char[value.length()+1];
		for (int i=1; i<value.length()-1; i++) {
			if (value[i]=='\\') {
				i++;
				switch (value[i]) {
				case 'r':
					tmp[j++] = '\r';
					break;
				case 'n':
					tmp[j++] = '\n';
					break;
				case 't':
					tmp[j++] = '\t';
					break;
				default:
					tmp[j++] = value[i];
					break;
				}
			} else {
				tmp[j++] = value[i];
			}
		}
		tmp[j] = '\0';
		token.value = tmp;
		delete [] tmp;
	} else if (isdigit(value[0])||value[0]=='.') {
		token.type = TokenNumber;
	} else {
		token.type = TokenSymbol;
		for (int j=0;j<value.length(); j++)
			token.value[j] = toupper(token.value[j]);
		for (int i=0; i<sizeof(tokenStrings)/sizeof(tokenStrings[0]); i++)
			if (token.value == tokenStrings[i].value) {
				token.type = tokenStrings[i].token;
				break;
			}
	}
	currentTokenSet->push(token);
}


bool ShrimpScript::parseSyntax()
{
	ShrimpToken* tok;
	try {
		while (currentTokenSet->size()>0) {
			tok = getToken();
			switch (tok->type) {
			case TokenEndOfLine:
				popToken();
				break;
			case TokenDIM:
				popToken();
				parseDIM(0);
				break;
			case TokenCONST:
				popToken();
				parseCONST(0);
				break;
			case TokenIMPORT:
				popToken();
				parseIMPORT();
				break;
			case TokenFUNCTION:
				popToken();
				parseFUNCTION();
				break;
			default:
				setLastError("unknown word", tok->value, tok->line);
				return false;
			}
		}
	} catch (ShrimpSyntaxException e) {
		setLastError(e.getError(), e.getDetail(), e.getLine());
		return false;
	}
	for (map<string, ShrimpSyntaxNode*>::iterator it=functions.begin(); it!=functions.end(); it++ ) {
		ShrimpSyntaxNode* node = it->second;
		if (node&&node->line==-1) {
			setLastError("Function not implemented", it->first);
			return false;
		}
	}
	return true;
}

void ShrimpScript::parseDIM(ShrimpSyntaxNode* func)
{
	ShrimpToken* tok;
	ShrimpVariable* var;
	map<string, ShrimpVariable*> *varList;
	bool	readVar = true;
	for (;;) {
		tok = getToken();
		if (readVar && tok->type == TokenSymbol) {
			if (func==0)
				varList = &(this->globals);
			else
				varList = &(func->localVariables);
			if ((*varList)[tok->value])
				throw ShrimpSyntaxException("Redim variable", tok->value, tok->line);
			var = new ShrimpVariable();
			if (func==0) {
				var->global = 1;
			} else {
				var->global = 0;
				var->index = func->variableCount ++;
			}
			(*varList)[tok->value] = var;
			readVar = false;
		} else if (tok->type == TokenComma) {
			readVar = true;
		} else if (tok->type == TokenEndOfLine) {
			break;
		} else {
			throw ShrimpSyntaxException("Syntax error", tok->value, tok->line);
		}
		popToken();
	}
}

ShrimpToken* ShrimpScript::getToken()
{
	if (currentTokenSet->size()==0)
		throw ShrimpSyntaxException("Syntax not finished", "");
	return &currentTokenSet->front();
}

void ShrimpScript::parseFUNCTION()
{
	ShrimpToken *tok = getToken();
	ShrimpVariable* var;
	ShrimpSyntaxNode* func;
	if (tok->type!=TokenSymbol)
		throw ShrimpSyntaxException("Syntax error", tok->value, tok->line);
	if (func=functions[tok->value]) {
		if (func->line!=-1)
			throw ShrimpSyntaxException("Function redefined", tok->value, tok->line);
	} else {
		func = new ShrimpSyntaxNode(0);
	}
	func->line = tok->line;
	func->type = ShrimpFunction;
	func->function = func;
	func->variableCount = 0;
	functions[tok->value] = func;
	func->symbolName = tok->value;
	popToken();
	tok = getToken();
	if (tok->type!=TokenEndOfLine) {
		if (tok->value!="(")
			throw ShrimpSyntaxException("Syntax error, missing (", tok->value, tok->line);
		popToken();
		for (;;) {
			tok = getToken();
			if (tok->type==TokenOperator&&tok->value==")")
				break;
			if (tok->type==TokenSymbol) {
				if (func->localVariables[tok->value])
					throw ShrimpSyntaxException("Parameter redefined", tok->value, tok->line);
				func->localVariables[tok->value] = var = new ShrimpVariable();
				var->index = func->variableCount ++;
			} else if (tok->type==TokenComma) {
			} else {
				throw ShrimpSyntaxException("Syntax error, unknown word", tok->value, tok->line);
			}
			popToken();
		}
		popToken();
	}
	endLine();
	// Return Variable
	getVariable(func, func->symbolName, true);
	// Parse function body
	parseBody(func);
	tok = getToken();
	if (tok->type!=TokenFUNCTION)
		throw ShrimpSyntaxException("Missing END FUNCTION", tok->value, tok->line);
	popToken();
	endLine();
}

void ShrimpScript::parseBody(ShrimpSyntaxNode* parent)
{
	ShrimpToken *tok;
	ShrimpSyntaxNode* pre = 0, *p;
	for (;;) {
		tok = getToken();
		if (tok->type==TokenEndOfLine) {
			popToken();
			continue;
		}
		if (tok->type==TokenEND) {
			popToken();
			return;
		}
		p = new ShrimpSyntaxNode(parent, pre, 0, tok->line, tok->value);
		if (pre)
			pre->right = p;
		switch (tok->type) {
		case TokenDIM:
			popToken();
			parseDIM(parent);
			delete p;
			p = 0;
			break;
		case TokenIF:
			p->type = ShrimpCondition;
			popToken();
			p->expression = parseExpression(p);
			tok = getToken();
			if (tok->type!=TokenTHEN)
				throw ShrimpSyntaxException("Syntax error, missing THEN", tok->value, tok->line);
			popToken();
			endLine();
			{
				ShrimpSyntaxNode* node = new ShrimpSyntaxNode(p);
				p->conditionTrue = node;
				node->type = ShrimpScope;
				parseBody(node);
			}
			tok = getToken();
			if (tok->type==TokenELSE) {
				popToken();
				endLine();
				ShrimpSyntaxNode* node = new ShrimpSyntaxNode(p);
				p->conditionFalse = node;
				node->type = ShrimpScope;
				parseBody(node);
			} else
				p->conditionFalse = 0;
			tok = getToken();
			if (tok->type!=TokenIF)
				throw ShrimpSyntaxException("Syntax error, missing END IF", tok->value, tok->line);
			popToken();
			break;
		case TokenFOR: {
			ShrimpSyntaxNode* init, *step, *stepExpr, *expr;
			p->type = ShrimpLoop;
			popToken();
			tok = getToken();
			p->symbolName = tok->value;
			popToken();
			tok = getToken();
			if (tok->value != "=")
				throw ShrimpSyntaxException("Syntax error, missing '='", tok->value, tok->line);
			popToken();
			//Parse initialization
			init = new ShrimpSyntaxNode(parent);
			init->type = ShrimpAssignment;
			init->symbolName = p->symbolName;
			init->value = getVariable(parent, p->symbolName, true);
			init->expression = parseExpression(parent);
			parent->sentences.push_back(init);
			tok = getToken();
			if (tok->type != TokenTO)
				throw ShrimpSyntaxException("Syntax error, missing 'TO'", tok->value, tok->line);
			popToken();
			//Parse expression
			expr = parseExpression(parent);
			//Parse Step
			tok = getToken();
			step = new ShrimpSyntaxNode(parent);
			step->type = ShrimpAssignment;
			step->symbolName = init->symbolName;
			step->value = init->value;
			if (tok->type == TokenSTEP) {
				popToken();
				stepExpr = parseExpression(parent);
			} else {
				stepExpr = new ShrimpSyntaxNode(parent);
				stepExpr->type = ShrimpConst;
				stepExpr->value = new ShrimpVariable();
				stepExpr->value->setInteger(1);
			}
			ShrimpSyntaxNode* nodeVar = new ShrimpSyntaxNode(parent),
			*nodeP = new ShrimpSyntaxNode(parent);
			nodeVar->type = ShrimpVar;
			nodeVar->symbolName = init->symbolName;
			nodeVar->value = init->value;
			nodeP->type = ShrimpEval;
			nodeP->operation = TokenAdd;
			nodeP->left = nodeVar;
			nodeP->right = stepExpr;
			step->expression = nodeP;
			//Parse body
			endLine();
			ShrimpSyntaxNode* body = new ShrimpSyntaxNode(parent);
			body->type = ShrimpScope;
			p->conditionTrue = body;
			parseBody(body);
			//parse next
			tok = getToken();
			if (tok->type != TokenNEXT)
				throw ShrimpSyntaxException("Syntax error, missing 'NEXT'", tok->value, tok->line);
			popToken();
			tok = getToken();
			if (tok->type==TokenSymbol && tok->value!=p->symbolName )
				throw ShrimpSyntaxException("Syntax error, missing 'FOR' for NEXT", tok->value, tok->line);
			body->sentences.push_back(step);
			nodeVar = new ShrimpSyntaxNode(parent);
			nodeP = new ShrimpSyntaxNode(parent);
			nodeVar->type = ShrimpVar;
			nodeVar->value = init->value;
			nodeVar->symbolName = init->symbolName;
			nodeP->type = ShrimpEval;
			if (stepExpr->type==ShrimpConst && stepExpr->value->getInteger()<0)
				nodeP->operation = TokenLargerOrEqual;
			else
				nodeP->operation = TokenLessOrEqual;
			nodeP->left = nodeVar;
			nodeP->right = expr;
			p->expression = nodeP;
			break;
		}
		case TokenDO: { //Do While ...\n...\nLoop  or Do\n...\nLoop Until...
			ShrimpSyntaxNode* body;
			int method = 0;
			popToken();
			tok = getToken();
			if (tok->type==TokenUNTIL)
				method = 1;
			else if (tok->type==TokenWHILE)
				method = 2;
			else if ( tok->type!=TokenEndOfLine)
				throw ShrimpSyntaxException("Syntax error, missing 'WHILE' or 'UNTIL' for 'DO'", tok->value, tok->line);
			popToken();
			if (method) {
				p->expression = parseExpression(parent);
				endLine();
			}
			body = new ShrimpSyntaxNode(parent);
			body->type = ShrimpScope;
			parseBody(body);
			//parse loop
			tok = getToken();
			if ( tok->type != TokenLOOP )
				throw ShrimpSyntaxException("Syntax error, missing 'LOOP' for 'DO'", tok->value, tok->line);
			popToken();
			if (!method) {
				tok = getToken();
				if (tok->type==TokenUNTIL)
					method = 1;
				else if (tok->type==TokenWHILE)
					method = 2;
				else if ( tok->type!=TokenEndOfLine)
					throw ShrimpSyntaxException("Syntax error, missing 'WHILE' or 'UNTIL' for 'LOOP'", tok->value, tok->line);
				popToken();
				p->expression = parseExpression(parent);
			}
			if (method==1) {
				p->conditionFalse = body;
				p->conditionTrue = 0;
			} else {
				p->conditionTrue = body;
				p->conditionFalse = 0;
			}
			p->type = ShrimpLoop;
			break;
		}
		case TokenWHILE: { //While ... \n...\nWend
			ShrimpSyntaxNode* body;
			popToken();
			p->expression = parseExpression(parent);
			body = new ShrimpSyntaxNode(parent);
			body->type = ShrimpScope;
			parseBody(body);
			p->conditionTrue = body;
			tok = getToken();
			if (tok->type!=TokenWEND)
				throw ShrimpSyntaxException("Syntax error, missing 'WEND' for 'WHILE'", tok->value, tok->line);
			popToken();
			p->type = ShrimpLoop;
			break;
		}
		case TokenSELECT: { //Select
			ShrimpSyntaxNode* expr, *left, *body, *right, *cond, *eq, *exitCase, *orCase;
			p->type = ShrimpSelect;
			p->line = tok->line;
			popToken();
			tok = getToken();
			if (tok->type!=TokenCASE)
				throw ShrimpSyntaxException("Syntax error, missing 'CASE' for 'SELECT'", tok->value, tok->line);
			popToken();
			expr = parseExpression(parent);
			endLine();
			for (;;) {
				tok = getToken();
				if (tok->type==TokenEND) {
					popToken();
					tok = getToken();
					if (tok->type!=TokenSELECT)
						throw ShrimpSyntaxException("Syntax error, missing 'SELECT' for 'END'", tok->value, tok->line);
					popToken();
					break;
				}
				if (tok->type!=TokenCASE)
					throw ShrimpSyntaxException("Syntax error, missing 'CASE'", tok->value, tok->line);
				popToken();
				right = parseExpression(parent);
				if (right->type!=ShrimpConst)
					throw ShrimpSyntaxException("Error: case value should be a constant", "", p->line);
				//build condition
				cond = new ShrimpSyntaxNode(parent);
				cond->type = ShrimpCondition;
				eq = new ShrimpSyntaxNode(parent);
				left = new ShrimpSyntaxNode(parent);
				*left = *expr;
				eq->type = ShrimpEval;
				eq->left = left;
				eq->right = right;
				eq->operation = TokenEqual;
				cond->expression = eq;
				tok = getToken();
				while (tok->type==TokenComma) { //Case 1,2,3
					popToken();
					right = parseExpression(parent);
					if (right->type!=ShrimpConst)
						throw ShrimpSyntaxException("Error: case value should be a constant", "", p->line);
					eq = new ShrimpSyntaxNode(parent);
					left = new ShrimpSyntaxNode(parent);
					*left = *expr;
					eq->type = ShrimpEval;
					eq->left = left;
					eq->right = right;
					eq->operation = TokenEqual;
					orCase = new ShrimpSyntaxNode(parent);
					orCase->type = ShrimpEval;
					orCase->operation = TokenOR;
					orCase->left = cond->expression;
					orCase->right = eq;
					cond->expression = orCase;
					tok = getToken();
				}
				endLine();
				body = new ShrimpSyntaxNode(parent);
				body->type = ShrimpScope;
				parseBody(body);
				exitCase = new ShrimpSyntaxNode(parent);
				exitCase->type = ShrimpExit;
				exitCase->operation = TokenSELECT;
				body->sentences.push_back(exitCase);
				cond->conditionTrue = body;
				p->sentences.push_back(cond);
				tok = getToken();
				if (tok->type==TokenSELECT) {
					popToken();
					break;
				}
			}
			delete expr;
			break;
		}
		case TokenEXIT:
			popToken();
			tok = getToken();
			switch (tok->type) {
			case TokenFOR:
			case TokenDO:
			case TokenFUNCTION:
				p->type = ShrimpExit;
				p->operation = tok->type;
				break;
			default:
				throw ShrimpSyntaxException("Syntax error, invalid procedure", tok->value, tok->line);
			}
			popToken();
			break;
		case TokenLET:
			popToken();
			tok = getToken();
			if (tok->type!=TokenSymbol)
				throw ShrimpSyntaxException("Syntax error, not a valid name", tok->value, tok->line);
		case TokenSymbol:
			for (;;) {
				p->symbolName = tok->value;
				popToken();
				tok = getToken();
				if (tok->type==TokenOperator && tok->value=="=") {
					p->type = ShrimpAssignment;
					p->value = getVariable(parent, p->symbolName, true);
					if (p->value->constant)
						throw ShrimpSyntaxException("Error, trying to assign a new value to a constant",
						                            tok->value, tok->line);
					popToken();
					p->expression = parseExpression(p);
				} else {
					p->type = ShrimpCall;
					parseCALL(p);
				}
				tok = getToken();
				if (tok->type==TokenEndOfLine)
					break;
				parent->sentences.push_back(p);
				p = new ShrimpSyntaxNode(parent);
				if (tok->type!=TokenComma)
					throw ShrimpSyntaxException("Error, should be endline or comma",
					                            tok->value, tok->line);
				popToken();
				tok = getToken();
				if (tok->type!=TokenSymbol)
					throw ShrimpSyntaxException("Error, should be symbol after comma",
					                            tok->value, tok->line);
			}
			break;
		default:
			delete p;
			return;
			//throw ShrimpSyntaxException("Syntax error, unknown word", tok->value, tok->line);
		}
		if (p) {
			parent->sentences.push_back(p);
			pre = p;
		}
		endLine();
	}
}

ShrimpVariable* ShrimpScript::getVariable(ShrimpSyntaxNode* p, string name, bool create)
{
	ShrimpVariable* var;
	p = p->function;
	if (p && (var=p->localVariables[name]))
		return var;
	if (var=this->globals[name])
		return var;
	if (p && create) {
		if (this->functions[name]&&p->symbolName!=name)
			throw ShrimpSyntaxException("Name conflicts against a function name.", name);
		var = new ShrimpVariable();
		var->index = p->variableCount ++;
		p->localVariables[name] = var;
		return var;
	}
	return 0;
}

ShrimpSyntaxNode* ShrimpScript::parseExpression(ShrimpSyntaxNode* p)
{
	ShrimpToken *tok;
	stack<string> opStack;
	stack<ShrimpSyntaxNode*> varStack;
	ShrimpVariable* var;
	ShrimpSyntaxNode* node;
	int line = p->line;
	bool lastIsOp = true;
	int needFix = 0;
	stack<bool> isOpStack;
	stack<int> needFixStack;
	for (;;) {
		bool needPop = true;
		tok = getToken();
		switch (tok->type) {
		case TokenString:
		case TokenNumber:
		case TokenSymbol:
			node = new ShrimpSyntaxNode(p);
			if (tok->type == TokenString) {
				var = new ShrimpVariable();
				var->type = TypeString;
				var->stringValue = tok->value;
				node->type = ShrimpConst;
				node->value = var;
			} else if (tok->type==TokenNumber) {
				bool floatNumber = false;
				var = new ShrimpVariable();
				for (int i=0; i<tok->value.length(); i++)
					if (tok->value[i]=='.') {
						floatNumber = true;
						break;
					}
				if (floatNumber) {
					var->type = TypeFloat;
					var->floatValue = atof(tok->value.c_str());
				} else {
					var->type = TypeInteger;
					var->integerValue = atoi(tok->value.c_str());
				}
				node->type = ShrimpConst;
				node->value = var;
			} else {
				var = getVariable(p, tok->value, false);
				if ( var ) {
					node->type = ShrimpVar;
					node->value = var;
					node->symbolName = tok->value;
				} else { // Is a function call
					node->type = ShrimpCall;
					node->symbolName = tok->value;
					popToken();
					parseCALL(node);
					needPop = false;
				}
			}
			varStack.push(node);
			/*
			 * needFix is used for solving the case of one or more negative or positive signs
			 * For example, 12 * -1
			 * -1 * ( -1 --- --3 )
			 */
			while (needFix>0) {
				var = new ShrimpVariable();
				var->setInteger(0);
				ShrimpSyntaxNode* t = new ShrimpSyntaxNode(p), *t2;
				t->type = ShrimpConst;
				t->value = var;
				t2 = varStack.top();
				varStack.pop();
				varStack.push(t);
				varStack.push(t2);
				varStack.push(buildOperation(opStack.top(), varStack, line));
				opStack.pop();
				needFix --;
			}
			lastIsOp = false;
			break;
		case TokenOperator:
		case TokenOR:
		case TokenAND:
		case TokenMOD:
		case TokenXOR:
		case TokenNOT: {
			if (tok->value==")") {
				while ( opStack.size()>0 && opStack.top()!="(" ) {
					varStack.push(buildOperation(opStack.top(), varStack, line));
					opStack.pop();
				}
				if (opStack.size()==0) {
					if (varStack.size()!=1)
						throw ShrimpSyntaxException("One or more operands are still in the stack", "", line);
					return varStack.top();
				}
				opStack.pop();
				if (isOpStack.size()>0) {
					lastIsOp = isOpStack.top();
					needFix = needFixStack.top();
					isOpStack.pop();
					needFixStack.pop();
					lastIsOp = false;
				}
				while (needFix>0) {
					var = new ShrimpVariable();
					var->setInteger(0);
					ShrimpSyntaxNode* t = new ShrimpSyntaxNode(p), *t2;
					t->type = ShrimpConst;
					t->value = var;
					t2 = varStack.top();
					varStack.pop();
					varStack.push(t);
					varStack.push(t2);
					varStack.push(buildOperation(opStack.top(), varStack, line));
					opStack.pop();
					needFix --;
				}
				break;
			}
			if (opStack.size()>0 && !lastIsOp && tok->value!="(") {
				int pri = getOperatorPriority(tok->value);
				int prePri = getOperatorPriority(opStack.top());
				if (pri >= prePri) { // Meet a low priority operator
					varStack.push(buildOperation(opStack.top(), varStack, line));
					opStack.pop();
				}
			}
			// For example,  4*-2
			if (tok->value!="(") {
				if (lastIsOp)
					needFix ++;
				lastIsOp = true;
			} else {
				isOpStack.push( lastIsOp );
				needFixStack.push(needFix );
				lastIsOp = true;
				needFix = 0;
			}
			opStack.push(tok->value);
			break;
		}
		default:
			while ( opStack.size()>0 ) {
				varStack.push(buildOperation(opStack.top(), varStack, line));
				opStack.pop();
			}
			if (varStack.size()!=1)
				throw ShrimpSyntaxException("One or more operands are still in the stack", "", line);
			return varStack.top();
		}
		if (needPop)
			popToken();
	}
}

int ShrimpScript::getOperatorPriority(string op)
{
	int count = sizeof(opPriority) / sizeof(opPriority[0]);
	for (int i=0; i<count; i++ ) {
		if (opPriority[i].op == op)
			return opPriority[i].pri;
	}
	return -1;
}

void ShrimpScript::parseCALL(ShrimpSyntaxNode* p)
{
	ShrimpSyntaxNode* func;
	ShrimpToken* tok = getToken();
	bool readExpr = true, bracket = false;
	if (!(func=functions[p->symbolName])) {
		functions[p->symbolName] = func = new ShrimpSyntaxNode(0);
		func->type = ShrimpFunction;
		func->line = -1;
	}
	p->expression = func;
	if (tok->value == "(" ) {
		bracket = true;
		popToken();
		tok = getToken();
		if (tok->value == ")" ) {
			popToken();
			return;
		}
	} else {
		if (tok->type==TokenEndOfLine)
			return;
	}
	for (;;) {
		tok = getToken();
		if (readExpr) {
			ShrimpSyntaxNode* node = parseExpression(p);
			p->parameters.push_back(node);
			readExpr = false;
		} else {
			if (bracket) {
				if (tok->value==")")
					break;
			} else {
				if (tok->type==TokenEndOfLine)
					return;
			}
			if (tok->type!=TokenComma)
				throw ShrimpSyntaxException("Missing ',' before", tok->value, tok->line);
			readExpr = true;
			popToken();
		}
	}
	popToken();
}

ShrimpSyntaxNode* ShrimpScript::buildOperation(string op, stack<ShrimpSyntaxNode*>& varStack, int line)
{
	if (varStack.size()==0)
		throw ShrimpSyntaxException("Syntax error: missing operand for", op, line);
	ShrimpSyntaxNode* node = new ShrimpSyntaxNode(varStack.top());
	node->type = ShrimpEval;
	node->right = varStack.top();
	varStack.pop();
	if (varStack.size()==0)
		throw ShrimpSyntaxException("Syntax error: missing operand for", op, line);
	node->left = varStack.top();
	varStack.pop();
//	cout << (node->left->value?node->left->value->getString():"") << " "
//		<< (node->right->value?node->right->value->getString():"") << " " << op << endl;
	int count = sizeof(opPriority) / sizeof(opPriority[0]), i;
	for (i=0; i<count; i++ ) {
		if (opPriority[i].op == op) {
			node->operation = opPriority[i].opCode;
			break;
		}
	}
	if ( i== count )
		throw ShrimpSyntaxException("Internal error: failed to determine the operator", op, line);
	if ( node->left->type == ShrimpConst && node->right->type == ShrimpConst ) {
		ShrimpVariable *t = new ShrimpVariable();
		*t = evaluate(node);
		node->type = ShrimpConst;
		node->value = t;
		remove(node->left);
		remove(node->right);
	}
	return node;
}

void ShrimpScript::endLine()
{
	ShrimpToken* tok = getToken();
	if (tok->type!=TokenEndOfLine)
		throw ShrimpSyntaxException("Need a new line at", tok->value, tok->line);
	popToken();
}

bool ShrimpScript::addFunction(string name, void (*func)(vector<ShrimpVariable>& stkVar, ShrimpVariable& result, int paraCount))
{
	toUpperString(name);
	if (functions[name] || globals[name]) {
		setLastError("function redefined", name);
		return false;
	}
	ShrimpSyntaxNode* node = new ShrimpSyntaxNode(0);
	functions[name] = node;
	node->type = ShrimpExternal;
	node->value = (ShrimpVariable*) func;
	node->symbolName = name;
	return true;
}

void ShrimpScript::invokeExternal(string name, int paraCount)
{
	if (!functions[name])
		return;

}

void ShrimpScript::toUpperString(string& str)
{
	for (int i=0; i<str.length(); i++ )
		str[i] = toupper(str[i]);
}

ShrimpVariable ShrimpScript::invokeFunction(string name, int paraCount)
{
	ShrimpVariable var;
	ShrimpSyntaxNode* func = functions[name];
	if (!func)
		throw ShrimpRuntimeException("Function not found", name, name);
	if (func->type == ShrimpExternal ) {
		((void (*)(vector<ShrimpVariable>& stkVar, ShrimpVariable& result, int paraCount))(func->value))
		(runtimeStack, var, paraCount);
		return var;
	} else if (func->type != ShrimpFunction )
		throw ShrimpRuntimeException("Not a valid function", name, name);
	if (paraCount > func->variableCount)
		throw ShrimpRuntimeException("Too many parameters", name, name);
	for (int i=0; i<func->variableCount-paraCount; i++)
		runtimeStack.push_back(var);
	for (int j=0; j<func->sentences.size(); j++) {
		if ( interpret(func->sentences[j]) >= ISExitFunction )
			break;
	}
	// Save Return Value
	var = getRuntimeVar(func, func->symbolName);
	for (int i=0; i<func->variableCount-paraCount; i++)
		runtimeStack.pop_back();
	return var;
}

int ShrimpScript::interpret(ShrimpSyntaxNode* p)
{
	int result = ISContinue;
	ShrimpSyntaxNode* func = p;
	switch (p->type) {
	case ShrimpFunction:
		break;
	case ShrimpAssignment:
		getRuntimeVar(p, p->symbolName) = evaluate(p->expression);
		break;
	case ShrimpCondition:
		if (evaluate(p->expression).getInteger()) {
			result = interpret(p->conditionTrue);
		} else if (p->conditionFalse) {
			result = interpret(p->conditionFalse);
		}
		break;
	case ShrimpLoop:
		if (p->conditionTrue) {
			while (evaluate(p->expression).getInteger()) {
				result = interpret(p->conditionTrue);
				if ( result >= ISExitLoop )
					break;
			}
		} else if (p->conditionFalse) {
			while (!evaluate(p->expression).getInteger()) {
				result = interpret(p->conditionFalse);
				if ( result >= ISExitLoop )
					break;
			}
		}
		if (result==ISExitLoop)
			result = ISContinue;
		break;
	case ShrimpSelect:
		for (int j=0; j<func->sentences.size(); j++) {
			result = interpret(func->sentences[j]);
			if ( result >= ISExitSelect )
				break;
		}
		if (result == ISExitSelect)
			result = ISContinue;
		break;
	case ShrimpScope:
		for (int j=0; j<func->sentences.size(); j++) {
			result = interpret(func->sentences[j]);
			if ( result > ISContinue )
				break;
		}
		break;
	case ShrimpExit:
		switch (p->operation) {
		case TokenSELECT:
			result = ISExitSelect;
			break;
		case TokenFOR:
		case TokenDO:
			result = ISExitLoop;
			break;
		case TokenFUNCTION:
			result = ISExitFunction;
			break;
		}
		break;
	case ShrimpVar:
		break;
	case ShrimpCall:
		call(p);
		break;
	case ShrimpEval:
		cout << "run eval " << p->symbolName << endl;
		break;
	default:
		throw ShrimpRuntimeException("Unknown syntax", p->symbolName, func->symbolName, p->line);
	}
	return result;
}

ShrimpVariable& ShrimpScript::getRuntimeVar(ShrimpSyntaxNode* p, const string& name)
{
	int i = 0;
	ShrimpSyntaxNode* func = p->function;
	ShrimpVariable* var;
	if (var=func->localVariables[name]) {
		int pos = func->variableCount - var->index;
		return runtimeStack[runtimeStack.size()-pos];
	} else if (var=this->globals[name]) {
		return *var;
	}
	throw ShrimpRuntimeException("variable not found", name, func->symbolName, func->line);
}

ShrimpVariable ShrimpScript::evaluate(ShrimpSyntaxNode* p)
{
	ShrimpVariable ret;
	ShrimpVariable left, right;
	if (p->type==ShrimpVar) {
		return getRuntimeVar(p, p->symbolName);
	}
	if (p->type==ShrimpConst) {
		return *p->value;
	}
	if (p->type==ShrimpCall) {
		return call(p);
	}
	if (p->type!=ShrimpEval)
		throw ShrimpRuntimeException("evaluate noneval sentence", p->symbolName, p->symbolName, p->line);
	if (p->left)
		left = evaluate(p->left);
	if (p->right)
		right = evaluate(p->right);
//	cout << "left " << left.getInteger() << " right " << right.getInteger() << " op " << p->operation << endl;
	switch (p->operation) {
	case TokenAdd:
		switch (left.type) {
		case TypeInteger:
			ret.setInteger(left.getInteger() + right.getInteger() );
			break;
		case TypeString:
			ret.setString(left.getString() + right.getString() );
			break;
		case TypeFloat:
			ret.setFloat(left.getFloat() + right.getFloat() );
			break;
		}
		break;
	case TokenSub:
		switch (left.type) {
		case TypeInteger:
		case TypeString:
			ret.setInteger(left.getInteger() - right.getInteger() );
			break;
		case TypeFloat:
			ret.setFloat(left.getFloat() - right.getFloat() );
			break;
		}
		break;
	case TokenMul:
		switch (left.type) {
		case TypeInteger:
		case TypeString:
			ret.setInteger(left.getInteger() * right.getInteger() );
			break;
		case TypeFloat:
			ret.setFloat(left.getFloat() * right.getFloat() );
			break;
		}
		break;
	case TokenDiv:
		switch (left.type) {
		case TypeInteger:
		case TypeString:
			ret.setInteger(left.getInteger() / right.getInteger() );
			break;
		case TypeFloat:
			ret.setFloat(left.getFloat() / right.getFloat() );
			break;
		}
		break;
	case TokenNOT:
		ret.setInteger( !right.getInteger() );
		break;
	case TokenAND:
		ret.setInteger( left.getInteger() & right.getInteger() );
		break;
	case TokenOR:
		ret.setInteger( left.getInteger() | right.getInteger() );
		break;
	case TokenEqual:
		if (left.type == TypeString || right.type == TypeString) {
			ret.setInteger( left.getString()==right.getString() );
		} else {
			ret.setInteger( left.getInteger()==right.getInteger() );
		}
		break;
	case TokenNotEqual:
		if (left.type == TypeString || right.type == TypeString) {
			ret.setInteger( left.getString()!=right.getString() );
		} else {
			ret.setInteger( left.getInteger()!=right.getInteger() );
		}
		break;
	case TokenMOD:
		ret.setInteger( left.getInteger() % right.getInteger() );
		break;
	case TokenLess:
		switch (left.type) {
		case TypeInteger:
		case TypeString:
			ret.setInteger(left.getInteger() < right.getInteger() );
			break;
		case TypeFloat:
			ret.setFloat(left.getFloat() < right.getFloat() );
			break;
		}
		break;
	case TokenLessOrEqual:
		switch (left.type) {
		case TypeInteger:
		case TypeString:
			ret.setInteger(left.getInteger() <= right.getInteger() );
			break;
		case TypeFloat:
			ret.setFloat(left.getFloat() <= right.getFloat() );
			break;
		}
		break;
	case TokenLarger:
		switch (left.type) {
		case TypeInteger:
		case TypeString:
			ret.setInteger(left.getInteger() > right.getInteger() );
			break;
		case TypeFloat:
			ret.setFloat(left.getFloat() > right.getFloat() );
			break;
		}
		break;
	case TokenLargerOrEqual:
		switch (left.type) {
		case TypeInteger:
		case TypeString:
			ret.setInteger(left.getInteger() >= right.getInteger() );
			break;
		case TypeFloat:
			ret.setFloat(left.getFloat() >= right.getFloat() );
			break;
		}
		break;
	case TokenXOR:
		ret.setInteger( left.getInteger() ^ right.getInteger() );
		break;
	}
	return ret;
}

void ShrimpScript::remove(ShrimpSyntaxNode* p)
{
	switch (p->type) {
	case ShrimpConst:
		delete p->value;
		break;
	case ShrimpEval:
		remove(p->left);
		remove(p->right);
		break;
	}
	delete p;
}

void ShrimpScript::parseCONST(ShrimpSyntaxNode* func)
{
	ShrimpToken* tok;
	ShrimpVariable* var;
	map<string, ShrimpVariable*> *varList;
	bool	readVar = true;
	for (;;) {
		tok = getToken();
		if (readVar && tok->type == TokenSymbol) {
			string varName = tok->value;
			if (func==0)
				varList = &(this->globals);
			else
				varList = &(func->localVariables);
			if ((*varList)[tok->value])
				throw ShrimpSyntaxException("Redefine variable", tok->value, tok->line);
			var = new ShrimpVariable();
			popToken();
			tok = getToken();
			if (tok->value != "=" )
				throw ShrimpSyntaxException("Syntax error, missing '='", tok->value, tok->line);
			popToken();
			ShrimpSyntaxNode* p = parseExpression(p);
			if (p->type!=ShrimpConst)
				throw ShrimpSyntaxException("Not a proper constant value", tok->value, tok->line);
			*var = *(p->value);
			remove(p);
			var->constant = 1;
			if (func==0) {
				var->global = 1;
			} else {
				var->global = 0;
				var->index = func->variableCount ++;
			}
			(*varList)[varName] = var;
			readVar = false;
		} else if (tok->type == TokenComma) {
			readVar = true;
			popToken();
		} else if (tok->type == TokenEndOfLine) {
			popToken();
			break;
		} else {
			throw ShrimpSyntaxException("Syntax error", tok->value, tok->line);
		}
	}
}

ShrimpVariable ShrimpScript::call(ShrimpSyntaxNode* p)
{
	queue<ShrimpVariable> q;
	ShrimpVariable ret;
	for (int i=0; i<p->parameters.size(); i++ )
		q.push( evaluate(p->parameters[i]) );
	for (int i=0; i<p->parameters.size(); i++ ) {
		runtimeStack.push_back(q.front());
		q.pop();
	}
	ret = invokeFunction(p->symbolName, p->parameters.size());
	for (int i=0; i<p->parameters.size(); i++ )
		runtimeStack.pop_back();
	return ret;
}

void ShrimpScript::popToken()
{
	currentTokenSet->pop();
}

void ShrimpScript::pushToken(ShrimpToken tok)
{
	currentTokenSet->push(tok);
}

void ShrimpScript::parseIMPORT()
{
	ShrimpToken* tok=getToken();
	string path = tok->value;
	popToken();
	for (int i=0; i<path.length(); i++)
		if (path[i]=='.')
			path[i] = '/';
	path += ".fb";
	if ( !load(path) )
		throw ShrimpSyntaxException(string("Failed to include the file ")+path, getLastError(), tok->line);
}
vector<ShrimpVariable>& ShrimpScript::getRuntimeStack()
{
	return this->runtimeStack;
}

