#include "sound.h"

namespace AntiKill
{

Sound::Sound()
{
	sample = 0;
	midi = 0;
}

Sound::~Sound()
{
}

}

const string& AntiKill::Sound::getName() const
{
	return name;
}

bool AntiKill::Sound::load(string file)
{
	if(file.find("wav")>0){
		sample = load_sample(file.c_str());
	}else{
		midi = load_midi(file.c_str());
	}
	if( !sample && !midi ){
		allegro_message("failed to load sample file %s.", file.c_str());
		return false;
	}
	return true;
}

void AntiKill::Sound::play()
{
	play(volume);
}

AntiKill::Sound& AntiKill::Sound::setName(string s)
{
	name = s;
	return *this;
}

void AntiKill::Sound::stop()
{
	if( sample )
		stop_sample(sample);
	if( midi )
		stop_midi();
}

void AntiKill::Sound::play(int vol)
{
	if( sample )
		play_sample(sample, vol, pan, pitch, 0);
	if( midi )
		play_midi(midi, 0);
}

int AntiKill::Sound::getPan() const
{
	return pan;
}

int AntiKill::Sound::getPitch() const
{
	return pitch;
}

int AntiKill::Sound::getVolume() const
{
	return volume;
}

AntiKill::Sound& AntiKill::Sound::setPan(int pan)
{
	this->pan = pan;
	return *this;
}

AntiKill::Sound& AntiKill::Sound::setPitch(int pitch)
{
	this->pitch = pitch;
	return *this;
}

AntiKill::Sound& AntiKill::Sound::setVolume(int vol)
{
	this->volume = vol;
	return *this;
}

