#include "weapon.h"
#include "game.h"
#include "bullet.h"
#include <iostream>

namespace AntiKill
{

Weapon::Weapon(Player* p, string file)
{
	file = "image/role/" + p->getRole() + "/" + file + ".bmp";
	this->loadImage(file.c_str());
	this->angle = 0;
	this->bulletCurrent = 0;
	this->bulletBackup = 0;
	this->reloadTime = 0;
	this->owner = p;
	this->fireInterval = 5;
	this->interval = 0;
	this->reloading = false;
}

Weapon::~Weapon()
{
}

}

void AntiKill::Weapon::draw(BITMAP* dest, int x, int y)
{
	if(angle>64&&angle<128+64)
		rotate_sprite_v_flip(dest, this->bitmap, x-this->width/2, y-this->height/2, itofix(this->angle) );
	else
		rotate_sprite(dest, this->bitmap, x-this->width/2, y-this->height/2, itofix(this->angle) );
}

int AntiKill::Weapon::getAngle() const
{
	return this->angle;
}

void AntiKill::Weapon::setTarget(int dx, int dy)
{
	float th = atan((float)dy/dx);
	float r = th / (2*3.1415926);
	this->angle = abs(r * 256);
	if(dx<0&&dy>=0)
		this->angle = 128 + this->angle; 
	else if(dx<0&&dy<0)
		this->angle = 128 - this->angle;
	else if(dx>0&&dy>0)
		this->angle = 256 - this->angle;
}

int AntiKill::Weapon::getBulletCurrent() const
{
	return this->bulletCurrent;
}

int AntiKill::Weapon::getBulletTotal() const
{
	return this->bulletBackup+this->bulletCurrent;
}

void AntiKill::Weapon::reduceBullet()
{
	if(this->bulletCurrent<=0){
		return ;
	}
	this->bulletCurrent --;
}


int AntiKill::Weapon::getReloadTime() const
{
	return this->reloadTime;
}

bool AntiKill::Weapon::fire(int fireX, int fireY)
{
	if(this->bulletCurrent==0){
		if(this->bulletBackup>0)
			reload();
		return false;
	}
	if(this->interval>0)
		return false;
	reduceBullet();
	this->interval = this->fireInterval;
	if(this->bulletCurrent==0 && this->bulletBackup>0){
		reload();
	}
	return true;
}

void AntiKill::Weapon::move()
{
	if(this->interval>0){
		this->interval --;
		if(this->interval==0){
			if(this->reloading){
				this->reloading = false;
				int a = this->bulletsPerClip-this->bulletCurrent;
				a = min(this->bulletBackup, a);
				this->bulletCurrent += a;
				this->bulletBackup -= a;
			}
		}
	}else{
		if(this->bulletCurrent==0&&this->bulletBackup>0&&!this->reloading)
			reload();
	}
	Object::move();
}

AntiKill::Player* AntiKill::Weapon::getOwner()
{
	return this->owner;
}

void AntiKill::Weapon::setOwner(Player* p)
{
	this->owner = p;
}

void AntiKill::Weapon::reload()
{
	this->reloading = true;
	this->interval = this->reloadTime;
}

void AntiKill::Weapon::addBullets(int v)
{
	this->bulletBackup += v;
}

